﻿
--------------------------------
-- A_Chat_Dialog_GetTopX --
--------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_GetTopX]' ))
	drop procedure dbo.[A_Chat_Dialog_GetTopX]
GO

create procedure dbo.[A_Chat_Dialog_GetTopX]
	@authorizeduserid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = ''
as
begin
		-- Достает диалоги по дате последнего сообщения с конца указанную пачку (по индексу, количество)
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#TopDialog' ))
			Drop table #TopDialog

		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#DialogCountMembers' ))
			Drop table #DialogCountMembers
				
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#PrivateDialogOpponent' ))
			Drop table #PrivateDialogOpponent
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#FoundDialogs' ))
			Drop table #FoundDialogs

		Create table #TopDialog (IdDialog uniqueidentifier, Type int)
		Create table #DialogCountMembers (IdDialog uniqueidentifier, CountMembers int)
		Create table #PrivateDialogOpponent (IdDialog uniqueidentifier, PrivateIdUser uniqueidentifier)
		Create table #FoundDialogs (IdDialog uniqueidentifier)
		
		Declare @sql nvarchar(4000)
		Declare @rowcount int
		
		if (( @search1 = '' ) and ( @search2 = '' ))
		begin

			-- Нарезаем нужный интервал из сортированного по дате
			-- Сначала ограничение. Корректировка Count
			Select @rowcount = Count(*) 
			From dbo.[A_Chat_Dialog] dlg
			Where dlg.IdDialog in (	Select IdDialog 
									From dbo.[A_Chat_Members] 
									Where IdUser = @authorizeduserid )

			if ( @rowcount < @start + @count )
				Set @count = case when ( @rowcount - @start > 0 ) then @rowcount - @start else 0 end

			-- Потом выборка идентификаторов
			if ( @count > 0 ) begin
	
				Set @sql = 'Insert into #TopDialog ( IdDialog, Type )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdDialog, Type
					From (
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' dlg.IdDialog, dlg.Type, dlg.LastDateUTC
							From dbo.[A_Chat_Dialog] dlg
							Where dlg.IdDialog in (	Select IdDialog 
													From dbo.[A_Chat_Members] 
													Where IdUser = @usr )
							Order by LastDateUTC desc ) a
					Order by LastDateUTC asc'

				Exec sp_executesql @sql, N'@usr uniqueidentifier', @usr = @authorizeduserid

			end
			
		end else begin
			
			if (( @search1 = '' ) or ( @search2 = '' )) begin

				Declare @search nvarchar(4000)
				if ( @search1 != '' )
					Set @search = @search1
				else
					Set @search = @search2

				-- Ищем диалоги с участием пользователя с фильтром по названию
				Insert into #FoundDialogs
					Select idDialog
					From 
					(
						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type != 1
								and ( dlg.Name like '%'+@search1+'%' )
						
						Union all

						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg 
									inner join dbo.[A_Chat_Members] m on m.IdDialog = dlg.IdDialog
									inner join dbo.A_Users users on m.IdUser = users.ID
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type = 1
								and m.IdUser != @authorizeduserid
								and ( users.Name like '%'+@search+'%' )
					) a

			end else begin
			
				-- Ищем диалоги с участием пользователя с фильтром по названию
				Insert Into #FoundDialogs	
					Select IdDialog
					From 
					(
						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type != 1
								and (( dlg.Name like '%'+@search1+'%' ) or ( dlg.Name like '%'+@search2+'%' ))

						Union all

						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg 
									inner join dbo.[A_Chat_Members] m on ( m.IdDialog = dlg.IdDialog ) and ( m.IdUser != @authorizeduserid )
									inner join dbo.A_Users users on users.ID = m.IdUser
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid)
								and dlg.Type = 1
								and (( users.Name like '%'+@search1+'%' ) or ( users.Name like '%'+@search2+'%' ))
					) a				
				
			end

			-- Нарезаем нужный интервал из сортированного по дате (уже по фильтру)
			-- Сначала ограничение. Корректировка Count
			Select @rowcount = Count(*) 
			From dbo.[A_Chat_Dialog] dlg
				Inner join #FoundDialogs f on f.IdDialog = dlg.IdDialog
	
			if ( @rowcount < @start + @count )
				Set @count = case when ( @rowcount - @start > 0 ) then @rowcount - @start else 0 end

			-- Потом выборка идентификаторов
			if ( @count > 0 ) begin

				Set @sql = 'Insert into #TopDialog ( IdDialog, Type )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdDialog, Type
					From (
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' dlg.IdDialog, dlg.Type, dlg.LastDateUTC
							From dbo.[A_Chat_Dialog] dlg
								Inner join #FoundDialogs f on f.IdDialog = dlg.IdDialog
							Order by LastDateUTC desc ) a
					Order by LastDateUTC asc'

				Exec sp_executesql @sql

			end

		end

		
		-- Сохраняем количество участников
		Insert Into #DialogCountMembers ( IdDialog, CountMembers )
			Select	m.IdDialog,
				count(m.IdUser) as CountMembers
			From dbo.[A_Chat_Members] as m
			Where m.IdDialog in ( Select IdDialog From #TopDialog )
			Group by m.IdDialog
		

		-- Сохраняем оппонента приватных бесед
		Insert Into #PrivateDialogOpponent ( IdDialog, PrivateIdUser )
			Select	m.IdDialog,
					m.IdUser
			From dbo.[A_Chat_Members] as m
			Where m.IdDialog in ( Select IdDialog From #TopDialog Where [Type] = 1 )
				and m.IdUser != @authorizeduserid


		-- Итоговая выборка
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.God,
				dlg.LastDateUTC,					
				dlg.IdLinkedObject,
				m.IdMessage as LastMessageId,
				m.IdUser as LastUserId,
				u.Login as LastUserLogin,
				u.Name as LastUserName,
				m.DateUTC as LastMessageDTUTC,
				m.Text as LastMessageText,
				Case When ( m.IdUser = @authorizeduserid ) then	--'from me'
							Case When ((
											select	count(*)
											from	dbo.[A_Chat_NewMessages]
											where	IdMessage in (select LastIdMessage from #TopDialog) 
													and IdDialog = dlg.IdDialog
										)
											>=
										(
											select	count(*)
											from	dbo.[A_Chat_Members]
											where	IdDialog = dlg.IdDialog
													and IdUser != @authorizeduserid
										))
									Then 1 Else 0 
						End
					Else null
					End as FromMeIsNew,
				Case When ( m.IdUser != @authorizeduserid ) then --'for me'
							Case When ( Select count(*) 
										From dbo.[A_Chat_NewMessages] 
										Where IdMessage in ( Select LastIdMessage From #TopDialog ) 
												and IdDialog = dlg.IdDialog 
												and IdUser = @authorizeduserid ) = 0 
									Then 0 Else 1 
						End
					Else null
					End ForMeIsNew,
				dcm.CountMembers,
				pdo.PrivateIdUser
		From #TopDialog td 
				Inner join dbo.[A_Chat_Dialog] dlg on dlg.IdDialog = td.IdDialog
				Inner join #DialogCountMembers dcm on dcm.IdDialog = td.IdDialog
				Left join dbo.[A_Chat_Messages] m on m.IdMessage = dlg.LastIdMessage
				Left join #PrivateDialogOpponent pdo on pdo.IdDialog = td.IdDialog
				Left join dbo.A_Users u on u.Id = m.IdUser
		Order by dlg.LastDateUTC Desc

		
		drop table #DialogCountMembers
		drop table #TopDialog
		drop table #PrivateDialogOpponent
		drop table #FoundDialogs
		
end
GO


---------------------------------
-- A_Chat_Messages_GetTopX --
---------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetTopX]' ))
	drop procedure dbo.[A_Chat_Messages_GetTopX]
GO

create procedure dbo.[A_Chat_Messages_GetTopX]
	@authorizeduserid	uniqueidentifier,
	@dialogid	uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя

		-- Является ли участником чата
		Declare @ismember int
		
		Select @ismember = Count(*)
		From dbo.A_Chat_Members
		Where IdDialog = @dialogid
			And IdUser = @authorizeduserid

		if ( @ismember = 0 )
			return

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		-- Фильтр для текста
		Declare @filtertext nvarchar(4000)
		Set @filtertext = ''	

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @filtertext = ''

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @filtertext = ' and (Text like ''%' + @search + '%'')'

		end else begin

			Set @filtertext = ' and (( Text like ''%' + @search1 + '%'' ) or ( Text like ''%' + @search2 + '%'' ))'

		end
		

		Declare @sql nvarchar(4000)

		-- Сначала ограничение (на последней странице меньше записей). 
		Declare @rowcount int			
		Set @sql = 'Select @rc = Count(*) 
					From dbo.[A_Chat_Messages]
					Where (IdDialog = @dlg)' + @filterdate + @filtertext
		exec sp_executesql @sql, N'@rc int OUTPUT, @dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', 
					@rc = @rowcount OUTPUT, @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		-- Корректировка @count 
		if ( @rowcount < @start + @count )
			Set @count = case when ( @rowcount - @start > 0 ) then @rowcount - @start else 0 end

		-- Потом выборка идентификаторов
		if ( @count > 0 ) begin

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2
			
				Insert into #interval ( IdMessage )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + @filtertext + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end

				
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages]
											where IdMessage = i.IdMessage 
												and IdDialog = @dialogid
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members]
											where IdDialog = @dialogid
												and IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = @dialogid 
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		Order by m.DateUTC Desc
		
		-- Файлы привязанные
		Select	IdMessage,
				Id,
				FilePath,
				FileSize
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO



----------------------------------------
-- A_Chat_Messages_GetTopXByOldMapper --
----------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetTopXByOldMapper]' ))
	drop procedure dbo.[A_Chat_Messages_GetTopXByOldMapper]
GO

create procedure dbo.[A_Chat_Messages_GetTopXByOldMapper]
	@authorizeduserid	uniqueidentifier,
	@opponentid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя
	

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier, IdDialog uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		-- Фильтр для текста
		Declare @filtertext nvarchar(4000)
		Set @filtertext = ''	

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @filtertext = ''

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @filtertext = ' and (Text like ''%' + @search + '%'')'

		end else begin

			Set @filtertext = ' and (( Text like ''%' + @search1 + '%'' ) or ( Text like ''%' + @search2 + '%'' ))'

		end


		Declare @sql nvarchar(4000)

		-- Сначала ограничение (на последней странице меньше записей). 
		Declare @rowcount int			
		Set @sql = 'Select @rc = Count(*) 
					From dbo.[A_Chat_Messages] msg
						Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
					Where (msg.IdUser = @author)' + @filterdate + @filtertext
		exec sp_executesql @sql, N'@rc int, @author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@rc = @rowcount OUTPUT, @author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		-- Корректировка @count 
		if ( @rowcount < @start + @count )
			Set @count = case when ( @rowcount - @start > 0 ) then @rowcount - @start else 0 end

		-- Потом выборка идентификаторов
		if ( @count > 0 ) begin

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog, msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
							Where (msg.IdUser = @author)' + @filterdate + @filtertext + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end 
	
		
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages] nm
											where nm.IdMessage = i.IdMessage 
												and nm.IdDialog = i.IdDialog
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members] mem
											where mem.IdDialog = i.IdDialog
												and mem.IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = i.IdDialog
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		Order by m.DateUTC Asc
		
		-- Файлы привязанные
		Select	IdMessage,
				Id,
				FilePath,
				FileSize
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO

------------------------------------
-- A_Chat_Message_MarkViewed --
------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Message_MarkViewed]' ))
	drop procedure dbo.[A_Chat_Message_MarkViewed]
GO

create procedure dbo.[A_Chat_Message_MarkViewed]
	@authorizeduserid uniqueidentifier,
	@dialogid uniqueidentifier,
	@messageid	uniqueidentifier
as
begin
		-- Отмечает сообщение как прочитанное

		Declare @authorid uniqueidentifier, @markedcount int

		-- Был ли просмотрен кем либо уже ранее?
		if
		(
			(
				select	count(*)
				from	dbo.[A_Chat_NewMessages]
				where	IdMessage = @messageid
			)
				>=
			(
				select	count(*) - 1
				from	dbo.[A_Chat_Members]
				where	IdDialog = @dialogid
			)
		)
		begin

			select	@authorid = IdUser
			from	dbo.[A_Chat_Messages]
			where	IdMessage = @messageid		

		end

		
		-- Удаление всех предыдущих нужно по диалогу
		Declare @dt datetime

		Select @dt = [DateUTC]
		From dbo.[A_Chat_NewMessages]
		Where	IdUser = @authorizeduserid
				and IdMessage = @messageid				
		
		---- Помечаем все более ранние по диалогу
		Delete
		from dbo.[A_Chat_NewMessages]
		where	IdUser = @authorizeduserid
				and IdDialog = @dialogid
				and DateUTC <= @dt
				
		-- Сколько прочитано итого
		Set @markedcount = @@rowcount
		
		Select @authorid, @markedcount
end
GO


------------------------------------------
-- A_Chat_Message_MarkViewedByOldMapper --
------------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Message_MarkViewedByOldMapper]' ))
	drop procedure dbo.[A_Chat_Message_MarkViewedByOldMapper]
GO

create procedure dbo.[A_Chat_Message_MarkViewedByOldMapper]
	@authorizeduserid uniqueidentifier,
	@messageid	uniqueidentifier
as
begin
		-- Отмечает сообщение как прочитанное (для поддержки старой схемы в толстом клиенте с маппером из предыдущего недиалогового режима)

		-- Удаление всех предыдущих нужно и по диалогу, и по юзеру
		Declare @dt datetime, @dialogid uniqueidentifier, @authorid uniqueidentifier, @markedcount int
		
		Select @dt = m.[DateUTC], @dialogid = m.[IdDialog], @authorid = m.[IdUser]
		From dbo.A_Chat_NewMessages nm
			Inner join dbo.A_Chat_Messages m on m.IdMessage = nm.IdMessage
		Where	nm.IdUser = @authorizeduserid
				and m.IdMessage = @messageid

		---- Помечаем все ранние по диалогу (Не так идет рассылка маппером)
		--Delete
		--from dbo.[A_Chat_NewMessages]
		--where	IdUser = @authorizeduserid
		--		and IdDialog = @dialogid
		--		and DateUTC <= @dt

		-- Помечаем по юзеру-отправителю (Именно так идет рассылка маппером)
		---- Из приватной беседы с отправителем
		--Declare @privatedialogid uniqueidentifier
		--
		--/* в приватных всегда два участника */
		--Select @privatedialogid = d1.IdDialog
		--From dbo.A_Chat_Dialog d1
		--	Inner join dbo.A_Chat_Members m1 on m1.IdDialog = d1.IdDialog and m1.IdUser = @authorizeduserid
		--	Inner join dbo.A_Chat_Members m2 on m2.IdDialog = d1.IdDialog and m2.IdUser = @authorid
		--Where d1.Type = 1
		--
		--Delete
		--From dbo.[A_Chat_NewMessages]
		--Where	IdUser = @authorizeduserid
		--		and IdDialog = @privatedialogid
		--		and DateUTC <= @dt

		-- Из всех диалогов, но только те сообщения, что отправитель писал
		--Select * 
		--From dbo.A_Chat_NewMessages nm
		--	Inner join dbo.A_Chat_Messages m on m.IdMessage = nm.IdMessage
		--Where nm.IdUser = @authorizeduserid 
		--	and m.DateUTC <= @dt
		--	and m.IdUser = @authorid
			
		Delete
		From dbo.A_Chat_NewMessages
		Where IdUser = @authorizeduserid 
			and DateUTC <= @dt
			and IdMessage in ( Select IdMessage 
								From dbo.A_Chat_Messages m
								Where m.IdUser = @authorid )
								
		Set @markedcount = @@rowcount

		-- Возвращаем отправителя и ид чата
		Select @authorid, @dialogid, @markedcount
end
GO


----------------------------------------------------
If exists ( Select * From sysobjects where id = object_id ( 'A_Chat_Members' ))
	If not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_Chat_Members' 
							and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'IdAddedUser' )
	Alter Table dbo.A_Chat_Members
	Add [IdAddedUser] uniqueidentifier

GO


Update m
Set m.IdAddedUser = d.God
From A_Chat_Members m,
	A_Chat_Dialog d
Where ( m.IdDialog = d.IdDialog )
	and ( m.IdAddedUser is NULL )
GO

----------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Chat_Members_Get' ))
	Drop procedure dbo.A_Chat_Members_Get
GO

----------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Chat_Members_Add' ))
	Drop procedure dbo.A_Chat_Members_Add
GO

Create procedure [dbo].[A_Chat_Members_Add]
	@dialogid	uniqueidentifier,
	@member	uniqueidentifier,
	@addeduserid uniqueidentifier = '00000000-0000-0000-0000-000000000000'	
as
begin
		-- Добавляет указанного участника в чат

		Insert into dbo.[A_Chat_Members] ( IdDialog, IdUser, IdAddedUser )
		Values ( @dialogid, @member, @addeduserid )		

end
GO


----------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Chat_Dialog_Get' ))
	Drop procedure dbo.A_Chat_Dialog_Get
GO

Create procedure [dbo].[A_Chat_Dialog_Get]
	@dialogid	uniqueidentifier
as
begin
		-- Возвращает серверную инфу по конкретному чату
	
		-- Выборка 1 (данные чата)
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.LastIdMessage,
				dlg.LastDateUTC,
				dlg.IdLinkedObject,
				dlg.God
		From dbo.[A_Chat_Dialog] dlg
		Where dlg.IdDialog = @dialogid

		-- Выборка 2 (участники)
		Select m.IdDialog, m.IdUser, m.IdAddedUser 
		From dbo.A_Chat_Members m 
		Where m.IdDialog = @dialogid
end
GO


----------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Chat_Dialogs_GetTop' ))
	Drop procedure dbo.A_Chat_Dialogs_GetTop
GO

Create procedure [dbo].[A_Chat_Dialogs_GetTop]
	@countsec int,
	@dateutc datetime = null
as
begin
		-- Возвращает список горячих чатов для сервера по дате последнего сообщения, находящейся в некотором интервале.
		
		-- Определяем интервал и чаты попадающие в него

		if (@dateutc is null) 
			set @dateutc = getutcdate()

		Declare @datefrom datetime
		Set @datefrom = DateAdd ( second, -@countsec, @dateutc )

		Declare @t table ( id uniqueidentifier )
		
		Insert into @t
			Select	dlg.IdDialog
			From [dbo].[A_Chat_Dialog] dlg
			Where dlg.LastDateUTC >= @datefrom


		-- Выборка 1 (чаты)		
		
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.LastIdMessage,
				dlg.LastDateUTC,
				dlg.IdLinkedObject,
				dlg.God
		from [dbo].[A_Chat_Dialog] dlg
				inner join @t t on t.id = dlg.IdDialog

		-- Выборка 2 (участники)

		Select m.IdDialog, m.IdUser, m.IdAddedUser
		From A_Chat_Members m 
			inner join @t t on t.Id = m.IdDialog
		Order by m.IdDialog
		
end
GO