﻿if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_GetForUserByObject' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_GetForUserByObject
GO

Create procedure dbo.A_Policy_SubjectPrivileges_GetForUserByObject
	@iduser uniqueidentifier,
	@idobject uniqueidentifier = '00000000-0000-0000-0000-000000000000',
	@idprivilege uniqueidentifier = NULL,
	@onlyreturncount bit = 0,
	@resultcount int = 0 OUTPUT
as 
begin
	-- Возвращает список всех установленных привилегий (разрешений)
	--  на конкретный объект для указанного пользователя
	--  с учетом вложенности назначенных привилегий по разным объектам

	-- Обновляем кэш при необходимости
	exec A_Policy_SubjectGroupCache_CheckUpdate
	
	
	-- Строим разрешения на объект с учетом приоритета для субъектных групп (например всем разрешено, а юзеру запрещено)		
		
	-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
	if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
		Drop table #A
		
	Create table #A ( [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
		
	-- С учетом запретов и приоритетов разных субъектных групповых объединений
	-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
	Insert into #A ( [idPrivilege], [subjType], [value] )
		Select sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
		From A_Policy_SubjectPrivileges sp
			Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
		Where (sp.[idObject] = @idobject)
			and (ur.idUser = @iduser)
			and ((@idprivilege is NULL) or (sp.[idPrivilege] = @idprivilege))
		Group by sp.[idPrivilege], ur.[subjType]
				
	-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
	--  На выход нас интересуют разрешенные - те у кого value получился 1
	Declare @tempresult table ( [idobject] uniqueidentifier, [idprivilege] uniqueidentifier, [code] nvarchar(200) collate database_default, [type] int )
	
	Insert into @tempresult ( [idobject], [idprivilege], [code], [type] )		
		Select @idobject, a.[idPrivilege], p.[code], o.[type]
		From ( Select a.[idPrivilege], Min(b.[priority]) as priority
				From #A a
					inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
				Group by a.[idPrivilege]
			)x
			Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
			Inner join #A a on (b.[idType] = a.[subjType])
								and (x.[idPrivilege] = a.[idPrivilege])		
			Inner join dbo.A_Policy_Objects o on (o.[id] = @idobject)
			Inner join dbo.A_Policy_Privileges p on p.[id] = a.[idPrivilege]
		Where a.[value] = 1		
	
	-- Разные варианты возврата значения
	if ( @onlyreturncount = 0 ) begin	
		Select *
		From @tempresult	
	end else 	
		Select @resultcount = Count(*)
		From @tempresult
			
end
GO


--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_GetForUserByObjectType' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_GetForUserByObjectType
GO

Create procedure dbo.A_Policy_SubjectPrivileges_GetForUserByObjectType
	@iduser uniqueidentifier,
	@objecttype int = 0,
	@idparentproject uniqueidentifier = NULL,
	@idprivilege uniqueidentifier = NULL,
	@onlyreturncount bit = 0,
	@resultcount int = 0 OUTPUT
as 
begin
	-- Возвращает список всех установленных привилегий (разрешений) 
	--  для всех объектов указанного типа для указанного пользователя
	--  с учетом вложенности назначенных привилегий по разным объектам

	-- Обновляем кэш при необходимости
	exec A_Policy_SubjectGroupCache_CheckUpdate
	
	
	-- Строим разрешения на объект с учетом приоритета для субъектных групп (например всем разрешено, а юзеру запрещено)		
	
	-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
	if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
		Drop table #A
		
	Create table #A ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
		
	-- С учетом запретов и приоритетов разных субъектных групповых объединений
	-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
	Insert into #A ( [idObject], [idPrivilege], [subjType], [value] )
		Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
		From A_Policy_SubjectPrivileges sp
			Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
			Inner join dbo.A_Policy_Objects o on o.[id] = sp.[idObject]
		Where (ur.[idUser] = @iduser)
			and ((@objecttype = 0) or (o.[type] = @objecttype))
			and ((@idparentproject is NULL) or (o.[idParentProject] = @idparentproject))
			and ((@idprivilege is NULL) or (sp.[idPrivilege] = @idprivilege))
		Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
				
	-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
	--  На выход нас интересуют разрешенные - те у кого value получился 1
	Declare @tempresult table ( [idobject] uniqueidentifier, [idprivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	
	Insert into @tempresult ( [idobject], [idprivilege], [code] )		
		Select a.[idObject], a.[idPrivilege], p.[code]
		From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
				From #A a
					inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
				Group by a.[idObject], a.[idPrivilege]
			)x
			Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
			Inner join #A a on (b.[idType] = a.[subjType]) 
								and (x.[idObject] = a.[idObject]) 
								and (x.[idPrivilege] = a.[idPrivilege])
			Inner join dbo.A_Policy_Privileges p on p.[id] = a.[idPrivilege]
		Where a.[value] = 1
		Order by a.[idObject], a.[idPrivilege]
	
	
	-- Разные варианты возврата значения
	if ( @onlyreturncount = 0 ) begin	
		Select *
		From @tempresult	
	end else 	
		Select @resultcount = Count(*)
		From @tempresult
	
end
GO

--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_CheckUserHasModifyPrivilege' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_CheckUserHasModifyPrivilege
GO

Create procedure dbo.A_Policy_SubjectPrivileges_CheckUserHasModifyPrivilege
	@idUser uniqueidentifier,
	@idObject uniqueidentifier,
	@onlyreturnvalue bit = 0,
	@resultvalue int = 0 OUTPUT
as
begin
	-- Проверяет наличие у пользователя на объект привилегии, позволяющей модифицировать политику доступа (с учетом назначения разных субъектов)
	-- В выборке возвращает:
	--   0 - у объекта есть привилегия изменения прав, и у пользователя она установлена
	--   100 - объект не найден в таблице правовой политики
	--   102 - у объекта есть привилегия изменения прав, но у пользователя она не установлена
	--   1 - у объекта нет ни одной привилегии изменения прав
	
	-- Обновляем кэш при необходимости
	exec A_Policy_SubjectGroupCache_CheckUpdate
	
	
	-- Есть ли у объекта привилегии с типом isPolicyModify
	-- Если есть - смотрим, доступна ли она указанному пользователю - если не доступна - на выход
	-- Если доступна, или если нет привилегии с типом isPolicyModify, 
	-- то проверяем изменяемую привилегию. Даем изменять те, которые у самого есть.
	
	-- Тип объекта
	Declare @objType int	
	Select @objType = [type]
	From A_Policy_Objects o		
	Where o.[id] = @idObject
	
	-- Объект не найден
	if ( @objType is NULL ) begin
		if ( @onlyreturnvalue = 0 )
			Select 100
		else
			Set @resultvalue = 100
		return
	end
	
	
	-- Привилегии для объекта с типом 'модификация политики привилегий'
	Declare @pmPrivileges table ( [id] uniqueidentifier )
	Insert into @pmPrivileges ( [id] )
		Select p.[id]
		From A_Policy_Privileges p
		Where (p.[objType] = @objType)
			and (p.[isPolicyModify] = 1)
	
	
	-- Проверяем наличие привилегии изменения прав доступа у пользователя. Ищем хотя бы одну, которая разрешает
	if ( @@rowcount > 0 ) begin
		
		Declare @hasmodify int
		
		-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
		if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
			Drop table #A
			
		Create table #A ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
			
		-- С учетом запретов и приоритетов разных субъектных групповых объединений
		-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
		Insert into #A ( [idObject], [idPrivilege], [subjType], [value] )
			Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
			From A_Policy_SubjectPrivileges sp
				Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
				Inner join @pmPrivileges pmp on sp.[idPrivilege] = pmp.[id]
			Where (sp.[idObject] = @idObject)
				and (ur.[idUser] = @iduser)
			Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
					
		-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
		--  На выход нас интересуют разрешенные - те у кого value получился 1
		Select @hasmodify = Count ( * )
		From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
				From #A a
					inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
				Group by a.[idObject], a.[idPrivilege]
			)x
			Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
			Inner join #A a on (b.[idType] = a.[subjType]) 
								and (x.[idObject] = a.[idObject]) 
								and (x.[idPrivilege] = a.[idPrivilege])
		Where a.[value] = 1
		
		
		if (( @hasmodify is NULL ) or ( @hasmodify = 0 )) begin			
			-- Нельзя менять
			if ( @onlyreturnvalue = 0 )
				Select 102
			else
				Set @resultvalue = 102
			return
		end
		
		-- Можно менять
		if ( @onlyreturnvalue = 0 )
			Select 0
		else
			Set @resultvalue = 0
		return
		
	end
	
	if ( @onlyreturnvalue = 0 )
		Select 1
	else
		Set @resultvalue = 1
	
end
GO

--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy
GO

Create procedure dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy
	@idUser uniqueidentifier,
	@idObject uniqueidentifier,
	@idPrivilege uniqueidentifier,
	@onlyreturnvalue bit = 0,
	@resultvalue int = 0 OUTPUT
as 
begin

	-- Возвращает результат проверки возможности для указанного пользователя
	-- измененять указанную привилегию указанного объекта (с учетом разных типов субъектов)	
	-- В выборке возвращает:
	--   0 - у объекта есть привилегия изменения прав, и у пользователя она установлена, 
	--       либо у объекта нет привилегии изменения прав, но у пользователя установлена привилегия, 
	--       которую он хочет изменить (у другого субъекта или у себя)
	--   100 - объект не найден в таблице правовой политики
	--   102 - у объекта есть привилегия изменения прав, но у пользователя она не установлена	
	--   103 - у объекта нет ни одной привилегии изменения прав, и у пользователя не установлена привилегия, 
	--         которую он хочет изменить (у другого субъекта или у себя)
	
	
	-- Первая часть аналогична A_Policy_SubjectPrivileges_CheckUserHasModifyPrivilege.
	--   Нельзя вложенно исполнять INSERT EXEC, поэтому копия


	-- Обновляем кэш при необходимости
	exec A_Policy_SubjectGroupCache_CheckUpdate
	
	
	-- Есть ли у объекта привилегии с типом isPolicyModify
	-- Если есть - смотрим, доступна ли она указанному пользователю - если не доступна - на выход
	-- Если доступна, или если нет привилегии с типом isPolicyModify, 
	-- то проверяем изменяемую привилегию. Даем изменять те, которые у самого есть.
	
	-- Тип объекта
	Declare @objType int	
	Select @objType = [type]
	From A_Policy_Objects o		
	Where o.[id] = @idObject
	
	-- Объект не найден
	if ( @objType is NULL ) begin
		if ( @onlyreturnvalue = 0 )
			Select 100
		else
			Set @resultvalue = 100
		return
	end
	
	
	-- Привилегии для объекта с типом 'модификация политики привилегий'
	Declare @pmPrivileges table ( [id] uniqueidentifier )
	Insert into @pmPrivileges ( [id] )
		Select p.[id]
		From A_Policy_Privileges p
		Where (p.[objType] = @objType)
			and (p.[isPolicyModify] = 1)
	
	
	-- Проверяем наличие привилегии изменения прав доступа у пользователя. Ищем хотя бы одну, которая разрешает
	if ( @@rowcount > 0 ) begin
		
		Declare @hasmodify int
		
		-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
		if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
			Drop table #A
			
		Create table #A ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
			
		-- С учетом запретов и приоритетов разных субъектных групповых объединений
		-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
		Insert into #A ( [idObject], [idPrivilege], [subjType], [value] )
			Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
			From A_Policy_SubjectPrivileges sp
				Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
				Inner join @pmPrivileges pmp on sp.[idPrivilege] = pmp.[id]
			Where (sp.[idObject] = @idObject)
				and (ur.[idUser] = @iduser)
			Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
					
		-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
		--  На выход нас интересуют разрешенные - те у кого value получился 1
		Select @hasmodify = Count ( * )
		From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
				From #A a
					inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
				Group by a.[idObject], a.[idPrivilege]
			)x
			Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
			Inner join #A a on (b.[idType] = a.[subjType]) 
								and (x.[idObject] = a.[idObject]) 
								and (x.[idPrivilege] = a.[idPrivilege])
		Where a.[value] = 1
		
		
		if (( @hasmodify is NULL ) or ( @hasmodify = 0 )) begin			
			-- Нельзя менять
			if ( @onlyreturnvalue = 0 )
				Select 102
			else
				Set @resultvalue = 102
			return
		end
		
		-- Можно менять
		if ( @onlyreturnvalue = 0 )
			Select 0
		else
			Set @resultvalue = 0		
		return
		
	end
	
	Declare @res int 
	Set @res = 1

	
	-- Дальше продолжение уникальное
	if ( @res = 0 ) begin
		
		-- Можно менять
		if ( @onlyreturnvalue = 0 )
			Select 0
		else
			Set @resultvalue = 0
		return
	
	end else if ( @res in (100, 102)) begin
		
		-- Невозможно определить наличие.
		-- Привилегия такая есть у объекта, но пользователю не разрешено.
		-- Значит менять нельзя
		if ( @onlyreturnvalue = 0 )
			Select @res
		else
			Set @resultvalue = @res
		return
		
	end else /* res = 1 */ begin
	
		-- Проверяем, установлена ли у самого юзера привилегия, которую пытается менять
		-- Возможно это условие в дальнейшем пригодится безусловно даже в том случае, если установлена привилегия изменения прав доступа
		
		Declare @hassame int
		
		-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
		if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
			Drop table #B
			
		Create table #B ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
			
		-- С учетом запретов и приоритетов разных субъектных групповых объединений
		-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
		Insert into #B ( [idObject], [idPrivilege], [subjType], [value] )
			Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
			From A_Policy_SubjectPrivileges sp
				Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
			Where (sp.[idObject] = @idObject)
				and (sp.[idPrivilege] = @idPrivilege)
				and (ur.[idUser] = @iduser)				
			Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
					
		-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
		--  На выход нас интересуют разрешенные - те у кого value получился 1
		Select @hassame = Count ( * )
		From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
				From #B a
					inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
				Group by a.[idObject], a.[idPrivilege]
			)x
			Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
			Inner join #B a on (b.[idType] = a.[subjType]) 
								and (x.[idObject] = a.[idObject]) 
								and (x.[idPrivilege] = a.[idPrivilege])
		Where a.[value] = 1
		
		if (( @hassame is NULL ) or ( @hassame = 0 )) begin
			-- Нельзя менять
			if ( @onlyreturnvalue = 0 )
				Select 103
			else
				Set @resultvalue = 103
			return
		end
		
		-- Можно менять
		if ( @onlyreturnvalue = 0 )
			Select 0
		else
			Set @resultvalue = 0
		return
	
	end
	
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_TaskManager_Project_Save' ))
	Drop procedure dbo.A_TaskManager_Project_Save
GO

Create procedure [dbo].[A_TaskManager_Project_Save]
	@id uniqueidentifier,
	@name nvarchar ( 100 ),
	@description nvarchar ( 255 ) = '',
	@datecreation DateTime,
	@idcreator uniqueidentifier,
	@mainscript nvarchar ( 500 ) = null,
	@shared bit = 1,
	@urestype int = 0
as
begin

	if ( exists ( Select [id] from dbo.A_TaskManager_Projects where [id] = @id )) begin
		
		/* Проверка прав на изменение
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @idcreator,
			@idobject = @id,
			@idprivilege = 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3',
			@onlyreturncount = 1,
			@resultcount = @checkcount OUTPUT
	
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end
		
		-- Обновляем существующий проект
		Update A_TaskManager_Projects
		Set	[Name] = @name,
			[Description] = @description,
			[UserResType] = @urestype
		Where Id = @id
	
	end else begin
	
		-- Новый проект
		Insert into A_TaskManager_Projects ( [Id], [Name], [Description], [DateCreation], [IdCreator], [MainScript], [UserResType] )
		Values ( @id, @name, @description, @datecreation, @idcreator, @mainscript, @urestype )
	
	end


	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 101,
		@name = @name,
		@idParentProject = @id,
		@actionUserId = @idcreator
	
	
	-- В базе кубов поправляем имя проекта
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	If Exists ( Select * FROM oktell_cc_temp..sysobjects where id = Object_Id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_Project' ) )
		Update oktell_cc_temp.dbo.A_Cube_CC_Cat_Project
		Set Name = @name
		Where Id = @id
	
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_TaskManager_Project_Delete' ))
	Drop procedure dbo.A_TaskManager_Project_Delete
GO

Create procedure [dbo].[A_TaskManager_Project_Delete]
	@id uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin

	if ( exists ( Select [id] from dbo.A_TaskManager_Projects where [id] = @id )) begin
	
		/* Проверка прав пользователя на удаление
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @iduser,
			@idobject = @id,
			@idprivilege = 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3',
			@onlyreturncount = 1,
			@resultcount = @checkcount OUTPUT			
		
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end


		-- Удаляем проект
		Delete from A_TaskManager_Projects
		Where Id = @id


		-- Удаляем списки
		Declare tcursor Cursor For
			Select id 
			From A_TaskManager_Lists
			Where ProjectId = @id 

		Declare @idlist uniqueidentifier

		Open tcursor
		Fetch Next From tcursor Into @idlist

		while ( @@FETCH_STATUS = 0 ) begin
			exec A_TaskManager_List_Delete @idlist

			Fetch Next From tcursor Into @idlist
		end

		Close tcursor
		Deallocate tcursor


		-- Удаляем задачи, которые к спискам не относятся
		Delete From A_TaskManager_Tasks 
		Where IdProject = @id


		-- очищаем список пользовательских состояний уникальных для проекта
		--   не надо, чтобы кубы можно было пересекать. когда нибудь потом...
		--exec A_TaskManager_UserResultsProject_Clear @id
		
		
		-- Из таблиц политики прав доступа к объектам
		exec A_Policy_Objects_Remove @id		
		-- Заказ на очистку таблиц политики доступа
		exec A_Policy_SetFilter
		
	end
	
	-- Все удачно
	Select 0

end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_TaskManager_Project_GetAllViewed' ))
	Drop procedure dbo.A_TaskManager_Project_GetAllViewed
GO

Create procedure [dbo].[A_TaskManager_Project_GetAllViewed]
	@iduser uniqueidentifier
as
begin
	
	-- Привилегии доступные пользователю по проектам
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)	
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @iduser,
			@objecttype = 101
			--@idparentproject = NULL,
			--@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
		
	-- Непосредственно проекты, доступные хотя бы по одной привилегии
	Select t.[Id], t.[Name], t.[Description], t.[DateCreation], t.[IdCreator], t.[MainScript], cast(1 as bit), t.[UserResType]
	From (
			Select Distinct [Id]
			From A_TaskManager_Projects t
				Inner join #privileges p on p.[idObject] = t.[Id]
		) a
		Inner join A_TaskManager_Projects t on t.[Id] = a.[Id]
	
	-- Набор привилегий для каждого из доступных проектов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_CallCenter_Report_Save' ))
	Drop procedure dbo.A_CallCenter_Report_Save
GO

Create Procedure [dbo].[A_CallCenter_Report_Save]
	@id uniqueidentifier,
	@stream image,
	@name nvarchar(2000) = NULL,
	@iduser uniqueidentifier = NULL
as
begin
	-- Сохраняет (создает или изменяет) отчет
	
	if ( exists ( Select [id] from dbo.A_CallCenter_Reports where [id] = @id )) begin
		
		-- Пользователь NULL - это 
		--    либо обновление отчета при старте системы из файла обновления
		--    либо старая версия к новой базе подцепилась
		--  и в том и в другом случае даем сохранить
		if ( not @iduser is NULL ) begin
				
			/* Проверка прав на изменение
			*/
			Declare @checkcount int
			
			exec A_Policy_SubjectPrivileges_GetForUserByObject
					@iduser = @iduser,
					@idobject = @id,
					@idprivilege = 'A7ECEDEA-76A3-4BBF-98DA-D05CD1B84F9D',
					@onlyreturncount = 1,
					@resultcount = @checkcount OUTPUT				
			
			if ( @checkcount = 0 ) begin
				-- Прав недостаточно
				Select 1
				return
			end
		end
		
		-- Обновление существующего отчета
		Update A_CallCenter_Reports
		Set [Name] = @name,
			[Stream] = @stream
		Where [Id] = @id
		
	end else begin
		
		-- Это новый отчет
		Insert into A_CallCenter_Reports ( [Id], [Name], [Stream] )
		Values ( @id, @name, @stream )
		
	end
	
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 103,
		@name = @name,
		@idParentProject = NULL,
		@actionUserId = @iduser
	
	
	-- Все удачно
	Select 0
	
end
GO



-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_CallCenter_Report_Delete' ))
	Drop procedure dbo.A_CallCenter_Report_Delete
GO

Create Procedure [dbo].[A_CallCenter_Report_Delete]
	@id uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin
	-- Удаляет отчет
	
	if ( exists ( Select [id] from dbo.A_CallCenter_Reports where [id] = @id )) begin
	
		/* Проверка прав пользователя на удаление
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @iduser,
			@idobject = @id,
			@idprivilege = 'A7ECEDEA-76A3-4BBF-98DA-D05CD1B84F9D',
			@onlyreturncount = 1,				
			@resultcount = @checkcount OUTPUT
		
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end


		-- Из отчетов
		Delete 
		From A_CallCenter_Reports
		Where [Id] = @id
		
		-- Из таблиц политики прав доступа к объектам
		exec A_Policy_Objects_Remove @id
		
	end
	
	-- Все удачно
	Select 0
	
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_CallCenter_Report_GetAllViewed' ))
	Drop procedure dbo.A_CallCenter_Report_GetAllViewed
GO

Create Procedure dbo.A_CallCenter_Report_GetAllViewed
	@iduser uniqueidentifier
as
begin
	-- Возвращает все отчеты, доступные для просмотра указанному пользователю

	-- Привилегии доступные пользователю по отчетам
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @iduser,
			@objecttype = 103
			--@idparentproject = NULL,
			--@idprivilege = '67139CAC-5C95-45DF-88A1-FBE8CC8F7BA4'
		
	-- Непосредственно отчеты, доступные хотя бы по одной привилегии
	Select t.[Id], DataLength(t.[Stream]) as Length, t.[Stream] 
	From (
			Select Distinct [Id]
			From A_CallCenter_Reports t
				Inner join #privileges p on p.[idObject] = t.[Id]
		) a
		Inner join A_CallCenter_Reports t on t.[Id] = a.[Id]
	
	-- Набор привилегий для каждого из доступных отчетов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_CallCenter_Indicator_Save' ))
	Drop procedure dbo.A_CallCenter_Indicator_Save
GO

Create Procedure [dbo].[A_CallCenter_Indicator_Save]
	@id uniqueidentifier,
	@stream image,
	@name nvarchar(2000) = NULL,
	@iduser uniqueidentifier = NULL
as
begin
	-- Сохраняет (создает или изменяет) индикатор
	
	if ( exists ( Select [id] from dbo.A_CallCenter_Indicators where [id] = @id )) begin
		
		-- Пользователь NULL - это 
		--    либо обновление отчета при старте системы из файла обновления
		--    либо старая версия к новой базе подцепилась
		--  и в том и в другом случае даем сохранить
		if ( not @iduser is NULL ) begin
		
			/* Проверка прав на изменение
			*/
			Declare @checkcount int
			
			exec A_Policy_SubjectPrivileges_GetForUserByObject
				@iduser = @iduser,
				@idobject = @id,
				@idprivilege = 'A2E397DF-40DB-4C80-BB2D-8CD49A29E09D',
				@onlyreturncount = 1,
				@resultcount = @checkcount OUTPUT
		
			if ( @checkcount = 0 ) begin
				-- Прав недостаточно
				Select 1
				return
			end
		end
		
		-- Обновление существующего индикатора
		Update A_CallCenter_Indicators
		Set [Name] = @name,
			[Stream] = @stream
		Where [Id] = @id
		
	end else begin
		
		-- Это новый индикатор
		Insert into A_CallCenter_Indicators ( [Id], [Name], [Stream] )
		Values ( @id, @name, @stream )
		
	end
	
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 104,
		@name = @name,
		@idParentProject = NULL,
		@actionUserId = @iduser
	
	
	-- Все удачно
	Select 0	
	
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_CallCenter_Indicator_Delete' ))
	Drop procedure dbo.A_CallCenter_Indicator_Delete
GO

Create Procedure [dbo].[A_CallCenter_Indicator_Delete]
	@id uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin
	-- Удаляет индикатор

	if ( exists ( Select [id] from dbo.A_CallCenter_Indicators where [id] = @id )) begin
	
		/* Проверка прав пользователя на удаление
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @iduser,
			@idobject = @id,
			@idprivilege = 'A2E397DF-40DB-4C80-BB2D-8CD49A29E09D',
			@onlyreturncount = 1,				
			@resultcount = @checkcount OUTPUT
		
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end


		-- Из индикаторов
		Delete 
		From A_CallCenter_Indicators
		Where [Id] = @id
		
		-- Из таблиц политики прав доступа к объектам
		exec A_Policy_Objects_Remove @id
		
	end
	
	-- Все удачно
	Select 0
	
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_CallCenter_Indicator_GetAllViewed' ))
	Drop procedure dbo.A_CallCenter_Indicator_GetAllViewed
GO

Create Procedure dbo.A_CallCenter_Indicator_GetAllViewed
	@iduser uniqueidentifier
as
begin
	-- Возвращает все инликаторы, доступные для просмотра указанному пользователю
	
	-- Привилегии доступные пользователю по индикаторам
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @iduser,
			@objecttype = 104
			--@idparentproject = NULL,
			--@idprivilege = '495DEB4D-D5FF-4C52-A219-541C3AEC6CAC'
		
	-- Непосредственно индикаторы, доступные хотя бы по одной привилегии
	Select t.[Id], DataLength(t.[Stream]) as Length, t.[Stream] 
	From (
			Select Distinct [Id]
			From A_CallCenter_Indicators t
				Inner join #privileges p on p.[idObject] = t.[Id]
		) a
		Inner join A_CallCenter_Indicators t on t.[Id] = a.[Id]
	
	-- Набор привилегий для каждого из доступных индикаторов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO

-------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_TaskManager_List_GetViewedByProject' ))
	Drop procedure dbo.A_TaskManager_List_GetViewedByProject
GO

Create procedure [dbo].A_TaskManager_List_GetViewedByProject
	@idproject uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin

	-- Возвращает информацию о всех таблицах абонентов, входящих в указанный проект
	
	-- Привилегии доступные пользователю по таблицам абонентов
	if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @iduser,
			@objecttype = 107
			--@idparentproject = NULL,
			--@idprivilege = '495DEB4D-D5FF-4C52-A219-541C3AEC6CAC'


	-- Таблицы абонентов
	Select t.[Id], t.[Name], t.[Description], t.[ProjectId], t.[Type], 
		t.[Query], t.[JoinedTable], t.[ColumnId], t.[ColumnsPhone], 
		t.[ColumnName], t.[ColumnsViewed], t.[ColumnsViewedDescription],
		t.[ColumnsSqlType], t.[ColumnsSqlTypeSize],
		t.[GetViewedFromTable],
		t.[DbConnId], t.[DbConnString], t.[DbConnMode], t.[DbProviderName]
	From (
			Select Distinct [Id]
			From A_TaskManager_Lists t
				Inner join #privileges p on p.[idObject] = t.[Id]
			Where t.[ProjectId] = @idproject
		) a
		Inner join A_TaskManager_Lists t on t.[Id] = a.[Id]
	Order By t.[Name]
	
	
	-- Набор привилегий для каждого из доступных индикаторов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO



--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy
GO

Create procedure dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy
	@idUser uniqueidentifier,
	@idObject uniqueidentifier,
	@idPrivilege uniqueidentifier,
	@onlyreturnvalue bit = 0,
	@resultvalue int = 0 OUTPUT
as 
begin

	-- Возвращает результат проверки возможности для указанного пользователя
	-- измененять указанную привилегию указанного объекта (с учетом разных типов субъектов)	
	-- В выборке возвращает:
	--   0 - у объекта есть привилегия изменения прав, и у пользователя она установлена, 
	--       либо у объекта нет привилегии изменения прав, но у пользователя установлена привилегия, 
	--       которую он хочет изменить (у другого субъекта или у себя)
	--   100 - объект не найден в таблице правовой политики
	--   102 - у объекта есть привилегия изменения прав, но у пользователя она не установлена	
	--   103 - у объекта нет ни одной привилегии изменения прав, и у пользователя не установлена привилегия, 
	--         которую он хочет изменить (у другого субъекта или у себя)
	
	
	-- Первая часть аналогична A_Policy_SubjectPrivileges_CheckUserHasModifyPrivilege.
	--   Нельзя вложенно исполнять INSERT EXEC, поэтому копия


	-- Обновляем кэш при необходимости
	exec A_Policy_SubjectGroupCache_CheckUpdate
	
	
	-- Есть ли у объекта привилегии с типом isPolicyModify
	-- Если есть - смотрим, доступна ли она указанному пользователю - если не доступна - на выход
	-- Если доступна, или если нет привилегии с типом isPolicyModify, 
	-- то проверяем изменяемую привилегию. Даем изменять те, которые у самого есть.
	
	-- Тип объекта
	Declare @objType int	
	Select @objType = [type]
	From A_Policy_Objects o		
	Where o.[id] = @idObject
	
	-- Объект не найден
	if ( @objType is NULL ) begin
		if ( @onlyreturnvalue = 0 )
			Select 100
		else
			Set @resultvalue = 100
		return
	end
	
	
	-- Привилегии для объекта с типом 'модификация политики привилегий'
	Declare @pmPrivileges table ( [id] uniqueidentifier )
	Insert into @pmPrivileges ( [id] )
		Select p.[id]
		From A_Policy_Privileges p
		Where (p.[objType] = @objType)
			and (p.[isPolicyModify] = 1)
	
	
	-- Проверяем наличие привилегии изменения прав доступа у пользователя. Ищем хотя бы одну, которая разрешает
	if ( @@rowcount > 0 ) begin
		
		Declare @hasmodify int
		
		-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
		if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
			Drop table #A
			
		Create table #A ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
			
		-- С учетом запретов и приоритетов разных субъектных групповых объединений
		-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
		Insert into #A ( [idObject], [idPrivilege], [subjType], [value] )
			Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
			From A_Policy_SubjectPrivileges sp
				Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
				Inner join @pmPrivileges pmp on sp.[idPrivilege] = pmp.[id]
			Where (sp.[idObject] = @idObject)
				and (ur.[idUser] = @iduser)
			Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
					
		-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
		--  На выход нас интересуют разрешенные - те у кого value получился 1
		Select @hasmodify = Count ( * )
		From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
				From #A a
					inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
				Group by a.[idObject], a.[idPrivilege]
			)x
			Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
			Inner join #A a on (b.[idType] = a.[subjType]) 
								and (x.[idObject] = a.[idObject]) 
								and (x.[idPrivilege] = a.[idPrivilege])
		Where a.[value] = 1
		
		
		if (( @hasmodify is NULL ) or ( @hasmodify = 0 )) begin			
			-- Нельзя менять
			if ( @onlyreturnvalue = 0 )
				Select 102
			else
				Set @resultvalue = 102
			return
		end
		
		-- Можно менять
		if ( @onlyreturnvalue = 0 )
			Select 0
		else
			Set @resultvalue = 0		
		return
		
	end
	
	Declare @res int 
	Set @res = 1

	
	-- Дальше продолжение уникальное
	if ( @res = 0 ) begin
		
		-- Можно менять
		if ( @onlyreturnvalue = 0 )
			Select 0
		else
			Set @resultvalue = 0
		return
	
	end else if ( @res in (100, 102)) begin
		
		-- Невозможно определить наличие.
		-- Привилегия такая есть у объекта, но пользователю не разрешено.
		-- Значит менять нельзя
		if ( @onlyreturnvalue = 0 )
			Select @res
		else
			Set @resultvalue = @res
		return
		
	end else /* res = 1 */ begin
	
		-- Проверяем, установлена ли у самого юзера привилегия, которую пытается менять
		-- Возможно это условие в дальнейшем пригодится безусловно даже в том случае, если установлена привилегия изменения прав доступа
		
		Declare @hassame int
		
		-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
		if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
			Drop table #B
			
		Create table #B ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
			
		-- С учетом запретов и приоритетов разных субъектных групповых объединений
		-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
		Insert into #B ( [idObject], [idPrivilege], [subjType], [value] )
			Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
			From A_Policy_SubjectPrivileges sp
				Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
			Where (sp.[idObject] = @idObject)
				and (sp.[idPrivilege] = @idPrivilege)
				and (ur.[idUser] = @iduser)				
			Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
					
		-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
		--  На выход нас интересуют разрешенные - те у кого value получился 1
		Select @hassame = Count ( * )
		From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
				From #B a
					inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
				Group by a.[idObject], a.[idPrivilege]
			)x
			Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
			Inner join #B a on (b.[idType] = a.[subjType]) 
								and (x.[idObject] = a.[idObject]) 
								and (x.[idPrivilege] = a.[idPrivilege])
		Where a.[value] = 1
		
		if (( @hassame is NULL ) or ( @hassame = 0 )) begin
			-- Нельзя менять
			if ( @onlyreturnvalue = 0 )
				Select 103
			else
				Set @resultvalue = 103
			return
		end
		
		-- Можно менять
		if ( @onlyreturnvalue = 0 )
			Select 0
		else
			Set @resultvalue = 0
		return
	
	end
	
end
GO


--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_GetByObject' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_GetByObject
GO

Create procedure dbo.A_Policy_SubjectPrivileges_GetByObject
	@idobject uniqueidentifier,
	@actionUserId uniqueidentifier = NULL	
as 
begin
	-- Возвращает список установленных привилегий (всех - и разрешенных, и запрещенных) для указанного объекта
	--  без учета вложенности привилегий по разным объектам
	
	-- Общие группы, названия из языковой таблицы
	Declare @groupnames table ( [id] int, [extraId] uniqueidentifier, [code] nvarchar(200) collate database_default, [name] nvarchar(500) collate database_default)

	Insert into @groupnames ( [id], [code], [name] ) Values ( 4, 'alloperators', 'Все операторы' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 7, 'allsupervisors', 'Все супервизоры' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 12, 'currentprojectoperators', 'Операторы текущего проекта' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 13, 'currentprojectsupervisors', 'Супервизоры текущего проекта' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 14, 'allusers', 'Все пользователи' )
	Insert into @groupnames ( [id], [extraId], [code], [name] ) Values ( 101, 'F876E650-7972-475A-AE96-BCCF831AF818', 'currentprojectviewers', 'Пользователи с правом просмотра текущего проекта' )
	Insert into @groupnames ( [id], [extraId], [code], [name] ) Values ( 101, 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3', 'currentprojectredactors', 'Пользователи с правом изменения текущего проекта' )

	Update g
	Set g.[name] = l.[langvalue]
	From @groupnames g, A_LangTexts l
	Where g.[code] = l.[langkey]
		
			
	-- Права пользователя на изменение привилегий
	Declare @resmodifyall int
	Declare @userprivileges table ( [idPrivilege] uniqueidentifier, [isHigherPrivilege] bit )
	
	Set @resmodifyall = 0
	if ( not @actionUserId is null ) begin
	
		/* Привилегия изменения прав для этого объекта есть?
		*/
		exec A_Policy_SubjectPrivileges_CheckUserHasModifyPrivilege 
			@idUser = @actionUserId, 
			@idObject = @idobject,
			@onlyreturnvalue = 1,
			@resultvalue = @resmodifyall OUTPUT
			
		-- Если нет, то выгребаем те привелегии, которые юзер сам имеет
		if ( @resmodifyall = 1 ) begin
		
			-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
			if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
				Drop table #A
				
			Create table #A ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
				
			-- С учетом запретов и приоритетов разных субъектных групповых объединений
			-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
			Insert into #A ( [idObject], [idPrivilege], [subjType], [value] )
				Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
				From A_Policy_SubjectPrivileges sp
					Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
				Where (sp.[idObject] = @idObject)
					and (ur.[idUser] = @actionUserId)				
				Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
						
			-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
			--  На выход нас интересуют разрешенные - те у кого value получился 1
			Insert into @userprivileges ( [idPrivilege], [isHigherPrivilege] )
				Select a.[idPrivilege], 0
				From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
						From #A a
							inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
						Group by a.[idObject], a.[idPrivilege]
					)x
					Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
					Inner join #A a on (b.[idType] = a.[subjType]) 
										and (x.[idObject] = a.[idObject]) 
										and (x.[idPrivilege] = a.[idPrivilege])
				Where a.[value] = 1
		end
		
	end
	
	

	-- Выборка итоговая
	Select 
		sp.[subjType],
	
		case when (( sp.[subjType] = 101 ) 
					and ( sp.[subjExtraId] in ('F876E650-7972-475A-AE96-BCCF831AF818', 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3')))
				then NULL
			else sp.[idSubject]
		end as idSubject,
		
		case when ( sp.subjType in ( 12, 13 ))
				then NULL
			else sp.[subjExtraId]
		end as subjPrivilegeId,
		
		sp.[idPrivilege], 
		sp.[value], 
		sp.[isDefault], 
		sp.[idCreator],
		
		cast ( case when ( @resmodifyall = 0 )
					then 0
					when ( @resmodifyall = 1 )
					then IsNull ( up.[isHigherPrivilege], 1 )
					else 1
				end as bit ) as isHigherPrivilege,				
		
		st.[name] as subjTypeName, 
		p.[name] as subjPrivilegeName,		
		
		case when (sp.[subjType] = 1)
				then IsNull ( xusers.Name, '' )
			 when (sp.[subjType] = 2)
				then IsNull ( xdepartments.[Name], '' )
			 when (sp.[subjType] = 3)			 
				then IsNull ( xroles.[Name], '' )
			 when (sp.[subjType] in (5, 8))
				then IsNull ( xtasks.[Name], '' )
			 when (sp.[subjType] in (6, 9, 10))
				then IsNull ( xprojs.[Name], '' )
			 when (sp.[subjType] in (4, 7, 12, 13, 14))
				then IsNull ( xgroupnames.[Name], '' )
			 when ((sp.[subjType] = 101) and (sp.[subjExtraId] = 'F876E650-7972-475A-AE96-BCCF831AF818'))
				then IsNull ( xgroupnames.[Name], '' )
			 when ((sp.[subjType] = 101) and (sp.[subjExtraId] = 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3'))
				then IsNull ( xgroupnames.[Name], '' )
			 when (sp.[subjType] = 101)
				then IsNull ( xobjs.[name], '' )--Здесь из кэша субъектных групп надо взять
			 else NULL
		end as subjGroupName,
		
		case when (sp.[subjType] = 101)
				then IsNull ( xobjtypes.[id], 0 )
			 else NULL
		end as subjObjectType,
		 
		case when (sp.[subjType] = 101)
				then IsNull ( xobjtypes.[name], '' )
			 else NULL
		end as subjObjectTypeName,
		
		case when (sp.[subjType] = 101)
				then IsNull ( xobjs.[name], '' )
			 else NULL
		end as subjObjectName
		 
	From dbo.A_Policy_SubjectPrivileges sp
		Inner join dbo.A_Policy_SubjectTypes as st on st.[id] = sp.[subjType]
		Left join dbo.A_Policy_Privileges as p on p.[id] = sp.[subjExtraId]
		Left join dbo.A_Users as xusers on (sp.[subjType] = 1) and (xusers.[Id] = sp.[idSubject])
		Left join dbo.A_Groups as xdepartments on (sp.[subjType] = 2) and (xdepartments.[Id] = sp.[idSubject])
		Left join dbo.A_UserRoles_Roles as xroles on (sp.[subjType] = 3) and (xroles.[Id] = sp.[idSubject])
		Left join @groupnames as xgroupnames on ((sp.[subjType] in (4, 7, 12, 13, 14)) and (xgroupnames.[id] = sp.[subjType]))
											 or ((sp.[subjType] = 101) and (xgroupnames.[id] = sp.[subjType]) and (xgroupnames.[extraId] = sp.[subjExtraId]))
		Left join dbo.A_TaskManager_Tasks as xtasks on (sp.[subjType] in (5, 8)) and (xtasks.[Id] = sp.[idSubject])
		Left join dbo.A_TaskManager_Projects as xprojs on (sp.[subjType] in (6, 9, 10)) and (xprojs.[Id] = sp.[idSubject])
		--Left join dbo.A_TaskManager_Projects as xprojsx on (sp.[subjType] in (12, 13)) and (xprojs.[Id] = sp.[subjExtraId])
		Left join dbo.A_Policy_Objects xobjs on (sp.[subjType] = 101) and (xobjs.[id] = sp.[idSubject])
		Left join dbo.A_Policy_ObjectTypes xobjtypes on (sp.[subjType] = 101) and (xobjtypes.[id] = xobjs.[type])
		--Left join со всеми таблицами объектов, поддерживающих POLICY, через xobjs.type и xobjs.id
		Left join @userprivileges up on up.[idPrivilege] = sp.[idPrivilege]
	Where (sp.[idObject] = @idobject)

end
GO


--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_Save' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_Save
GO

Create procedure dbo.A_Policy_SubjectPrivileges_Save
	@idObject uniqueidentifier,
	@idPrivilege uniqueidentifier,
	@subjType int,	
	@idSubject uniqueidentifier = NULL,
	@subjExtraId uniqueidentifier = NULL,	
	@value int = 1,
	@default bit = 0,
	@actionUserId uniqueidentifier = NULL
as 
begin
	-- Добавляет/изменяет привилегию субъекту с проверкой права на операцию	
	-- Изменять может только тот, у кого стоит привилегия isPolicyModify для этого объекта
	-- В выборке возвращает:
	--   0 - сохранено
	--   101 - не указанные пользователи не могут изменять права доступа
	--   100 - объект не найден в таблице правовой политики
	--   102 - у объекта есть привилегия изменения прав, но у пользователя она не установлена	
	--   103 - у объекта нет ни одной привилегии изменения прав, и у пользователя не установлена привилегия, 
	--         которую он хочет изменить (у другого субъекта или у себя)
	--   104 - криво исполнилась процедура проверки права изменения желаемой привилегии
	
	-- Неуказанные юзеры не могут менять
	if ( @actionUserId is NULL ) begin
		Select 101
		return
	end
	
	/* Проверяем наличие привилегии изменения политики, а также проверяем
	*/
	Declare @res int
	exec dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy 
		@idUser = @actionUserId, 
		@idObject = @idObject, 
		@idPrivilege = @idPrivilege,
		@onlyreturnvalue = 1,
		@resultvalue = @res OUTPUT
		
		
	if ( @res is NULL ) begin
		Select 104
		return
	end
	
	if ( @res > 0 ) begin
		Select @res
		return
	end
	
	
	-- Модификации обновляемых полей перед сохранением
	if ( @subjType in ( 12, 13 )) begin
	
		-- Ид проекта для объекта надо достать и сохранить в ExtraId
		Select @subjExtraId = IsNull ( idParentProject, @subjExtraId )
		From A_Policy_Objects o
		Where o.[id] = @idObject
	
	end	else if (( @subjType = 101 ) and ( @subjExtraId in ('F876E650-7972-475A-AE96-BCCF831AF818', 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3'))) begin
	
		-- Ид проекта для объекта надо достать и сохранить в idSubject
		Select @idSubject = IsNull ( idParentProject, @idSubject )
		From A_Policy_Objects o
		Where o.[id] = @idObject
		
	end
	
	if ( @idSubject is NULL )
		Set @idSubject = '00000000-0000-0000-0000-000000000000'
	
	
	-- Изменение
	Update A_Policy_SubjectPrivileges
	Set [value] = @value,
		--[isDefault] = @default,--Изменения признака дефолтной привилегии не происходит. ее удалить нельзя
		[idCreator] = @actionUserId
	Where ([idObject] = @idobject)
		and ([idPrivilege] = @idPrivilege)
		and ([idSubject] = @idSubject)
		and ([subjType] = @subjType)
		and ((@subjExtraId is NULL) or ([subjExtraId] = @subjExtraId))
			
	if ( @@rowcount = 0 ) begin
		Insert into A_Policy_SubjectPrivileges ( [idSubject], [subjType], [subjExtraId], [idObject], [idPrivilege], [value], [isDefault], [idCreator] )
		Values ( @idSubject, @subjType, @subjExtraId, @idObject, @idPrivilege, @value, @default, @actionUserId )
	end
	
	Select 0
	
end
GO


--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_Remove' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_Remove
GO

Create procedure dbo.A_Policy_SubjectPrivileges_Remove
	@idObject uniqueidentifier,
	@idPrivilege uniqueidentifier,
	@subjType int,
	@idSubject uniqueidentifier = NULL,
	@subjExtraId uniqueidentifier = NULL,	
	@actionUserId uniqueidentifier = NULL
as 
begin
	-- Удаляет привилегию у субъекта с проверкой права на операцию
	-- Изменять может только тот, у кого стоит привилегия isPolicyModify для этого объекта
	-- В выборке возвращает:
	--   0 - удалено
	--   101 - не указанные пользователи не могут изменять права доступа
	--   100 - объект не найден в таблице правовой политики
	--   102 - у объекта есть привилегия изменения прав, но у пользователя она не установлена	
	--   103 - у объекта нет ни одной привилегии изменения прав, и у пользователя не установлена привилегия, 
	--         которую он хочет изменить (у другого субъекта или у себя)
	--   104 - криво исполнилась процедура проверки права изменения желаемой привилегии
	
	-- Неуказанные юзеры не могут менять
	if ( @actionUserId is NULL ) begin
		Select 101
		return
	end
	
	/* Проверяем наличие привилегии изменения политики, а также проверяем
	*/
	Declare @res int
	exec dbo.A_Policy_SubjectPrivileges_CheckCanModifyPolicy 
		@idUser = @actionUserId, 
		@idObject = @idObject, 
		@idPrivilege = @idPrivilege,
		@onlyreturnvalue = 1,
		@resultvalue = @res OUTPUT
	
	
	if ( @res is NULL ) begin
		Select 104
		return
	end
	
	if ( @res > 0 ) begin
		Select @res
		return
	end
	
	-- Удаление
	Delete
	From A_Policy_SubjectPrivileges
	Where ([idObject] = @idobject)
		and ([idPrivilege] = @idPrivilege)
		and ([subjType] = @subjType)
		and ((@idSubject is NULL) or ([idSubject] = @idSubject))		
		and ((@subjExtraId is NULL) or ([subjExtraId] = @subjExtraId))
		
	Select 0
		
end
GO


-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_TaskManager_List_GetViewedByProject' ))
	Drop procedure dbo.[A_TaskManager_List_GetViewedByProject]
GO

Create procedure [dbo].[A_TaskManager_List_GetViewedByProject]
	@idproject uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin

	-- Возвращает информацию о всех таблицах абонентов, входящих в указанный проект
	
	-- Привилегии доступные пользователю по таблицам абонентов
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @iduser,
			@objecttype = 107
			--@idparentproject = NULL,
			--@idprivilege = '495DEB4D-D5FF-4C52-A219-541C3AEC6CAC'


	-- Таблицы абонентов
	Select t.[Id], t.[Name], t.[Description], t.[ProjectId], t.[Type], 
		t.[Query], t.[JoinedTable], t.[ColumnId], t.[ColumnsPhone], 
		t.[ColumnName], t.[ColumnsViewed], t.[ColumnsViewedDescription],
		t.[ColumnsSqlType], t.[ColumnsSqlTypeSize],
		t.[GetViewedFromTable],
		t.[DbConnId], t.[DbConnString], t.[DbConnMode], t.[DbProviderName],
		t.[ColumnOperator]
	From (
			Select Distinct [Id]
			From A_TaskManager_Lists t
				Inner join #privileges p on p.[idObject] = t.[Id]
			Where t.[ProjectId] = @idproject
		) a
		Inner join A_TaskManager_Lists t on t.[Id] = a.[Id]
	Order By t.[Name]
	
	
	-- Набор привилегий для каждого из доступных индикаторов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO



--------------------------------------
-- WO_Dashboards_Dashboards_GetList --
--------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboards_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Dashboards_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboards_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Доступные пользователю проекты по привилегии ЧТЕНИЕ
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#projects' ))
		Drop table #projects
	Create table #projects ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #projects ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 101,
			--@idparentproject = NULL,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'

	-- Доступные пользователю дашборды по привилегии ЧТЕНИЕ
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#dashboards' ))
		Drop table #dashboards
	Create table #dashboards ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #dashboards ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 105,
			--@idparentproject = NULL,
			@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863'

	
	Select	d.Id,
			d.IdProject,
			d.Name,
			d.Creator,
			d.CreationDate
	From	[dbo].[WO_Dashboards_Dashboards] d
		Inner join #dashboards p on p.IdObject = d.Id
	Where ( d.IdProject is NULL )
		or exists ( Select IdObject From #projects pr Where pr.IdObject = d.IdProject )
	Order by d.Name asc
	
end
GO


------------------------------------
-- WO_Dashboards_Projects_GetList --
------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Projects_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Projects_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Projects_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Доступные пользователю проекты по привилегии ЧТЕНИЕ
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#projects' ))
		Drop table #projects
	Create table #projects ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #projects ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 101,
			--@idparentproject = NULL,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
			
	
	-- Достаем инфу по проектам
	Select	p.Id,
			p.Name
	From [oktell_settings].[dbo].[A_TaskManager_Projects] p
		Inner join #projects pp on pp.idObject = p.Id
	Order by p.Name asc
	
end
GO



--------------------------------------
-- WO_Dashboards_Dashboards_GetList --
--------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboards_GetListPrivileges]'))
DROP procedure [dbo].[WO_Dashboards_Dashboards_GetListPrivileges]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboards_GetListPrivileges]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Возвращает информацию о привилегиях пользователя по всем дашбордам
	
	-- Привилегии доступные пользователю по дашбордам
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 105
			--@idparentproject = NULL,
			--@idprivilege = '495DEB4D-D5FF-4C52-A219-541C3AEC6CAC'


	-- Набор привилегий для каждого из доступных дашбордов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO


---------------------------------
-- WO_Dashboards_Dashboard_Get --
---------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Get]'))
DROP procedure [dbo].[WO_Dashboards_Dashboard_Get]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Get]
	@authorizeduserid	uniqueidentifier,
	@dashboardid    uniqueidentifier
as
begin
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#pp' ))
		Drop table #pp
	Create table #pp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default, [type] int)
	Insert into #pp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @projectid,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
	
	/* Привилегия чтения по дашборду есть?
	*/
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#dp' ))
		Drop table #dp
	Create table #dp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default, [type] int)
	Insert into #dp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @dashboardid,
			@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863'			
			
	
	-- Достаем из пересечения
	Select	d.Id,
			d.IdProject,
			d.Name,
			d.Creator,
			d.CreationDate
	From	[dbo].[WO_Dashboards_Dashboards] d
		Inner join #dp dp on dp.IdObject = d.Id
	Where (d.Id = @dashboardid)	
		and ((d.IdProject is NULL ) 
			or (exists (Select idObject From #pp pp Where pp.idObject = d.IdProject)))
	
end
GO

-----------------------------------------------------------------------------	
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Dashboards_CheckCanEdit' ))
	Drop procedure dbo.A_Dashboards_CheckCanEdit
GO

Create procedure dbo.A_Dashboards_CheckCanEdit
	@authorizeduserid uniqueidentifier,
	@dashboardid uniqueidentifier,
	@canedit bit output	
as
begin
	-- Проверяет возможность редактирования дашборда пользователем

	-- По умолчанию запрещено
	Set @canedit = 0

	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	--Select @projectid = d.IdProject
	--From [dbo].[WO_Dashboards_Dashboards] d	
	--Where d.Id = @dashboardid
	Select @projectid = o.IdParentProject
	From [dbo].[A_Policy_Objects] o
	Where o.Id = @dashboardid
			
	if (( not @projectid is NULL ) and ( @projectid != '00000000-0000-0000-0000-000000000000' )) begin
	
		/* Привилегия просмотра по проекту есть?
		*/
		Declare @rowcntp int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @projectid,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
			@onlyreturncount = 1,
			@resultcount = @rowcntp OUTPUT
			
		-- Просмотр проекта запрещен, значит запрещено редактирование дашборда
		if ( @rowcntp = 0 )
			return
	end
	
	/* Привилегия изменения по дашборду есть?
	*/
	Declare @rowcntd int
	
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		--@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863', --просмотр
		@idprivilege = 'C04D309A-159F-4F13-9C50-01B74945CAD5', --изменение
		@onlyreturncount = 1,
		@resultcount = @rowcntd OUTPUT
			
	-- Нет связки привилегии/объекта/юзера по совместному фильтру
	if ( @rowcntd = 0 )
		return
		
	Set @canedit = 1		
	
end
GO
