﻿if exists ( Select * FRom sysobjects where id = object_id ( 'dbo.A_Policy_Filter' ))
	Drop procedure dbo.A_Policy_Filter
GO

Create procedure [dbo].[A_Policy_Filter]
as 
begin
	-- Очищает остатки по удаленным объектам и субъектам из таблиц правовой политики
	
	-- Объекты
	Delete
	From dbo.A_Policy_Objects
	Where (([type] = 101) -- Projects
			and not exists ( Select o.[Id] From A_TaskManager_Projects o Where o.[Id] = A_Policy_Objects.[id] ))
		or (([type] = 102) -- Tasks
			and not exists ( Select o.[Id] From A_TaskManager_Tasks o Where o.[Id] = A_Policy_Objects.[id] )
			and not exists ( Select o.[Id] From A_SmsTaskManager_Tasks o Where o.[Id] = A_Policy_Objects.[id] ))
		or (([type] = 103) -- Reports
			and not exists ( Select o.[Id] From A_CallCenter_Reports o Where o.[Id] = A_Policy_Objects.[id] ))
		or (([type] = 104) -- Indicators
			and not exists ( Select o.[Id] From A_CallCenter_Indicators o Where o.[Id] = A_Policy_Objects.[id] ))
		--or (([type] = 106) -- Scripts
		--	and not exists ( Select o.[Id] From A_Scripts o Where o.[Id] = A_Policy_Objects.[id] ))
		or (([type] = 107) -- CallLists
			and not exists ( Select o.[Id] From A_TaskManager_Lists o Where o.[Id] = A_Policy_Objects.[id] ))
		
		
	-- Привилегии
	Delete
	From dbo.A_Policy_SubjectPrivileges
	Where not exists ( Select o.[Id] From A_Policy_Privileges o Where o.[id] = A_Policy_SubjectPrivileges.[idPrivilege] )
		
		
	-- Субъекты обычные
	Delete
	From dbo.A_Policy_SubjectPrivileges
	Where (([subjType] = 1) -- Users
			and not exists ( Select o.[Id] From A_Users o Where o.[Id] = A_Policy_SubjectPrivileges.[idSubject] ))
		or (([subjType] = 2) -- Groups
			and not exists ( Select o.[Id] From A_Groups o Where o.[Id] = A_Policy_SubjectPrivileges.[idSubject] ))
		or (([subjType] = 3) -- Roles
			and not exists ( Select o.[Id] From A_UserRoles_Roles o Where o.[Id] = A_Policy_SubjectPrivileges.[idSubject] ))					
		or (([subjType] in (5, 8)) -- TaskOperators, TaskSupervisors
			and not exists ( Select o.[Id] From A_TaskManager_Tasks o Where o.[Id] = A_Policy_SubjectPrivileges.[idSubject] ))
		or (([subjType] in (6, 9, 10)) -- ProjectOperators, ProjectSupervisors, ProjectOwners
			and not exists ( Select o.[Id] From A_TaskManager_Tasks o Where o.[Id] = A_Policy_SubjectPrivileges.[idSubject] ))
		or (([subjType] in (12, 13))-- CurrentProject Operators, Supervisors
			and not exists ( Select o.[Id] From A_TaskManager_Projects o Where o.[Id] = A_Policy_SubjectPrivileges.[subjExtraId] ))
			
			
	-- Субъекты в замыкании
	Delete
	From dbo.A_Policy_SubjectPrivileges
	Where ([subjType] = 101)
		and (( not exists ( Select o.[Id] From A_Policy_Objects o Where o.[id] = A_Policy_SubjectPrivileges.[idSubject] ))
			or ( not exists ( Select o.[Id] From A_Policy_Privileges o Where o.[id] = A_Policy_SubjectPrivileges.[subjExtraId] )))
			
	-- Кэш
	Delete
	From dbo.A_Policy_SubjectGroupCache
	Where not exists ( Select o.Id From A_Users o Where o.[Id] = A_Policy_SubjectGroupCache.[idUser] )
		or (([subjType] = 1) -- Users
			and not exists ( Select o.[Id] From A_Users o Where o.[Id] = A_Policy_SubjectGroupCache.[idSubject] ))
		or (([subjType] = 2) 
			and not exists ( Select o.Id From A_Groups o Where o.[Id] = A_Policy_SubjectGroupCache.[idSubject] ))
		or (([subjType] = 3) 
			and not exists ( Select o.Id From A_UserRoles_Roles o Where o.[Id] = A_Policy_SubjectGroupCache.[idSubject] ))
		or (([subjType] in (5, 8)) -- TaskOperators, TaskSupervisors
			and not exists ( Select o.[Id] From A_TaskManager_Tasks o Where o.[Id] = A_Policy_SubjectGroupCache.[idSubject] ))
		or (([subjType] in (6, 9, 10)) -- ProjectOperators, ProjectSupervisors, ProjectOwners
			and not exists ( Select o.[Id] From A_TaskManager_Tasks o Where o.[Id] = A_Policy_SubjectGroupCache.[idSubject] ))
		or (([subjType] in (12, 13))-- CurrentProject Operators, Supervisors
			and not exists ( Select o.[Id] From A_TaskManager_Projects o Where o.[Id] = A_Policy_SubjectGroupCache.[subjExtraId] ))
					
					
	-- Сопоставление с объектами в таблице типов
	Delete 
	From A_Policy_SubjectPrivileges
	WHere not exists ( Select Id From A_Policy_Objects o where o.Id = A_Policy_SubjectPrivileges.idObject )	
	
end
GO


if exists ( Select * FRom sysobjects where id = object_id ( 'dbo.A_Policy_CheckObjectExists' ))
	Drop procedure dbo.A_Policy_CheckObjectExists
GO

Create procedure [dbo].[A_Policy_CheckObjectExists]
	@idobject uniqueidentifier,
	@objecttype int
as
begin
	
	Declare @cnt int
	Set @cnt = 0
	
	-- Объекты
	if ( @objecttype = 101 ) begin
		
		Select @cnt = Count(*)
		From A_TaskManager_Projects 
		Where Id = @idobject
		
	end else if ( @objecttype = 102 ) begin
	
		Select @cnt = Count(*)
		From ( 
				Select Id
				From A_TaskManager_Tasks
				Where Id = @idobject
			
				Union
				
				Select Id
				From A_SmsTaskManager_Tasks
				Where Id = @idobject
			) a
	
	end else if ( @objecttype = 103 ) begin
		
		Select @cnt = Count(*)
		From A_CallCenter_Reports 
		Where Id = @idobject
		
	end else if ( @objecttype = 104 ) begin
		
		Select @cnt = Count(*)
		From A_CallCenter_Indicators
		Where Id = @idobject
		
	end else if ( @objecttype = 106 ) begin
		
		/*Scripts*/		
		Set @cnt = 0
		
	end else if ( @objecttype = 107 ) begin
	
		Select @cnt = Count(*)
		From A_TaskManager_Lists 
		Where Id = @idobject
		
	end
	
	Select 131106, @cnt
		
end
GO


if exists ( Select * FRom sysobjects where id = object_id ( 'dbo.A_Dashboards_CheckCanEdit' ))
	Drop procedure dbo.[A_Dashboards_CheckCanEdit]
GO
