﻿

IF OBJECT_ID('dbo.H_Authorizeduserid_SPID') IS NOT NULL
    DROP TABLE dbo.H_Authorizeduserid_SPID
GO


CREATE TABLE  dbo.H_Authorizeduserid_SPID 
(
	[authorizeduser_name] nVARCHAR(500) collate database_default, 
	[getdate] DATETIME default getdate(), 
	[tr_id] varchar(255) collate database_default, 
	[sql_text] nVARCHAR(4000) collate database_default
)
GO

CREATE INDEX  H_Authorizeduserid_SPID_tr_id_indx ON dbo.H_Authorizeduserid_SPID ([tr_id] DESC)
CREATE INDEX  H_Authorizeduserid_SPID_getdate_indx ON dbo.H_Authorizeduserid_SPID ([getdate] DESC)
GO


IF OBJECT_ID('dbo.H_HISTORY') IS NOT NULL
    DROP TABLE dbo.H_HISTORY
GO

CREATE TABLE [dbo].[H_HISTORY] (
    [id] uniqueidentifier,
    [type] varchar(10) collate database_default NULL,
    [table_name] nvarchar(100) collate database_default NULL,
    [authorizeduser_name] nvarchar(500) collate database_default NULL,
    [columns_name] nvarchar(4000) collate database_default NULL,
    [old_values] nvarchar(4000) collate database_default NULL,
    [new_values] nvarchar(4000) collate database_default NULL,
    [side] smallint NULL,
    [getdate] datetime NULL DEFAULT (getdate()),
    [full_query_transaction] nvarchar(4000) collate database_default NULL,
    [full_query_transactionT] nvarchar(4000) collate database_default NULL,
    [tr_id] varchar(255) collate database_default,
    [col_id] varchar(50) collate database_default
)

CREATE INDEX  H_HISTORY_type_getdate_indx ON H_HISTORY ([type], [getdate] DESC)
GO 


IF OBJECT_ID('dbo.H_Authorizeduserid_SPID_Save') IS NOT NULL
    DROP PROCEDURE dbo.H_Authorizeduserid_SPID_Save
GO
   
CREATE PROCEDURE dbo.H_Authorizeduserid_SPID_Save 
    @authorizeduserid uniqueidentifier = NULL
as
begin

    Declare 
		@authorizeduser_name nvarchar(500),
		@tr_id varchar(255), 
		@sql_text nvarchar(4000), 
		@sql_handle varbinary(255)
		
    Set @authorizeduser_name = COALESCE(
		(	Select [name] 
			From A_Users 
			Where id = @authorizeduserid
		), SUSER_SNAME())
    
    -- Ид сессии
    If ((Select cast(replace(left(cast(SERVERPROPERTY('productversion') as varchar),2),'.','') as int)) > 8) begin
        
        -- For MSSQLSERVER 2005-...
        Set @tr_id = (
			Select cast(transaction_id as varchar(255)) 
			From sys.dm_tran_current_transaction )
        
        Select top 1 @sql_handle = r.sql_handle
        From sys.dm_exec_requests r 
        Where r.transaction_id = @tr_id
                
        Select top 1 @sql_text = s_est.[text] 
        From sys.dm_exec_sql_text (@sql_handle) s_est
    
    end else begin
    
		-- For MSSQLSERVER 2000
        Declare
            @spid int,
			@stmt_start int,
			@stmt_end int

        Set @spid = @@SPID -- Fill this in
        Execute sp_getbindtoken @tr_id OUTPUT
        
        Select top 1
            @sql_handle = sql_handle,
			@stmt_start = case stmt_start when 0 then 0 else stmt_start / 2 end,
			@stmt_end = case stmt_end when -1 then -1 else stmt_end / 2 end
        From master.dbo.sysprocesses
        Where spid = @spid
        Order by ecid

        Select @sql_text = 
            Substring(text,
            		Coalesce(NullIf(@stmt_start, 0), 1),
            		Case @stmt_end
            			When -1
            				Then DATALENGTH(text)
            			Else
            				(@stmt_end - @stmt_start)
            		End
            	)
        FROM ::fn_get_sql(@sql_handle)
        
        Create table #temp (
			[EventType] nvarchar (30) collate database_default, 
			[Parameters] int, 
			[EventInfo] nvarchar(4000) 
		)
        
        Insert into #temp
			Exec sp_executesql N'DBCC inputbuffer (@@spid) WITH NO_INFOMSGS'
			
        Set @sql_text = COALESCE(NullIf(@sql_text,''), (select TOP 1 EventInfo from #temp))
    
    end
    
    Insert Into [dbo].H_Authorizeduserid_SPID ( [authorizeduser_name], [getdate], [tr_id], [sql_text])  
    Values (@authorizeduser_name, GetDate(), @tr_id, @sql_text)
    
END
GO
