﻿
-----------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.ParseJsonToLinkGuidGuid'))
	Drop function dbo.ParseJsonToLinkGuidGuid
GO

Create Function dbo.ParseJsonToLinkGuidGuid ( @json nvarchar(4000) )
returns @table table ( a uniqueidentifier, b uniqueidentifier )
as
begin
	/* 
	Производит преобразование json-строки в таблицу.
	Для целей вставки множественных связывающих значений Guid - Guid	
	
	Пример json строки, подлежащей парсингу
	{ 
		"11111111-1111-1111-1111-111111111111": [
			"15555555-5555-5555-5555-555555555555",
			"25555555-5555-5555-5555-555555555555"		
		],
		"22222222-2222-2222-2222-222222222222": [
			"15555555-5555-5555-5555-555555555555"
		]
	}
	*/

	Declare @a nvarchar(50), @b nvarchar(50), @temp nvarchar(4000)
	Declare @from int, @idx int, @idx0 int, @idx1 int, @idx2 int
	Set @from = 0
	
	while ( 1 = 1 ) begin
	
		Set @idx = PatIndex ( '%:%', Right(@json, Len(@json) - @from )) + @from
		if ( @idx <= @from )
			break
			
		-- Предшествующий ключ между кавычками
		Set @temp = Reverse ( Substring ( @json, @from, @idx - @from ))
		Set @idx1 = PatIndex ( '%"%', @temp )
		Set @temp = Right ( @temp, Len ( @temp ) - @idx1 )
		Set @idx1 = PatIndex ( '%"%', @temp )
		Set @a = Reverse ( Left ( @temp, @idx1 - 1 ))
		
		-- Последующие значения
		Set @from = @idx
		Set @idx1 = PatIndex ( '%[%', Right(@json, DataLength(@json) - @from )) + @from
		Set @idx2 = PatIndex ( '%]%', Right(@json, DataLength(@json) - @from )) + @from
				
		while ( 1 = 1 ) begin
			
			Set @from = @idx
			Set @idx = PatIndex ( '%,%', Right(@json, DataLength(@json) - @from )) + @from
			
			if ((@idx <= @idx1) or (@idx > @idx2)) begin 				
				-- Впереди больше нет запятых до ']'
				Set @idx = @idx2
			end 
				
			Set @temp = Reverse ( Substring ( @json, @from, @idx - @from ))
			Set @idx1 = PatIndex ( '%"%', @temp )
			Set @temp = Right ( @temp, Len ( @temp ) - @idx1 )
			Set @idx1 = PatIndex ( '%"%', @temp )
			Set @b = Reverse ( Left ( @temp, @idx1 - 1 ))
						
			Set @from = @idx + 1
						
			Insert into @table ( a, b )
			Values ( @a, @b )
			
			if ( @from >= @idx2 )
				break	
			
		end			
		
	end
	
	return
	
end
GO


-----------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.ParseXmlToLinkGuidGuid'))
	Drop procedure dbo.ParseXmlToLinkGuidGuid
GO

Create procedure dbo.ParseXmlToLinkGuidGuid 
	@xml ntext	
as
begin
	/* 
	Производит преобразование xml-строки в таблицу.
	Для целей вставки множественных связывающих значений Guid - Guid	
	
	Пример json строки, подлежащей парсингу
	<items>
		<a ida="11111111-1111-1111-1111-111111111111">
			<b idb="15555555-5555-5555-5555-555555555555"/>
			<b idb="15555555-5555-5555-5555-555555555555"/>
		</a>
		<a ida="22222222-2222-2222-2222-222222222222">
			<b idb="15555555-5555-5555-5555-555555555555"/>			
		</a>
	</items>	
	*/
	
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Select ida, idb		
	From OpenXml (@idoc, '/items/a/b', 2)	
	With ( 
		ida varchar(50) '../@ida',
		idb varchar(50) '@idb'
	)	
	
end
GO

-----------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_UserControls_Save' ))
	Drop procedure dbo.A_UserControls_Save
GO

Create procedure dbo.A_UserControls_Save
	@mode int, -- 1 - by UserA, 0 - by UserB
	@xml ntext, -- <items><a ida=""><b idb=""/><b idb=""/></a><a ida....>
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
	
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml ( a uniqueidentifier, b uniqueidentifier )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select ida, idb		
		From OpenXml (@idoc, '/items/a/b', 2)	
		With ( 
			ida varchar(50) '../@ida',
			idb varchar(50) '@idb'
		)	
		
		
	if ( @mode = 1 ) begin
		
		Delete
		From dbo.A_UserControls
		Where exists ( Select 1 From #tmp_xml x Where dbo.A_UserControls.UserA = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControls.UserA = x.a) and (A_UserControls.UserB = x.b) )
			
		Insert into dbo.A_UserControls (UserA, UserB)
			Select Distinct a, b
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControls u Where (u.UserA = x.a) and (u.UserB = x.b) )
			
	end else begin
			
		Delete
		From dbo.A_UserControls
		Where exists ( Select 1 From #tmp_xml x Where dbo.A_UserControls.UserB = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControls.UserB = x.a) and (A_UserControls.UserA = x.b) )
			
		Insert into dbo.A_UserControls (UserA, UserB)
			Select Distinct b, a
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControls u Where (u.UserA = x.b) and (u.UserB = x.a) )
	
	end
	
end
GO


-----------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_UserControlLines_Save' ))
	Drop procedure dbo.A_UserControlLines_Save
GO

Create procedure dbo.A_UserControlLines_Save
	@mode int, -- 1 - by User, 0 - by Line
	@xml ntext, -- <items><a ida=""><b idb=""/><b idb=""/></a><a ida....>
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid

	
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml ( a uniqueidentifier, b uniqueidentifier )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select ida, idb		
		From OpenXml (@idoc, '/items/a/b', 2)	
		With ( 
			ida varchar(50) '../@ida',
			idb varchar(50) '@idb'
		)	
		
		
	if ( @mode = 1 ) begin
		
		Delete
		From dbo.A_UserControlLines
		Where exists ( Select 1 From #tmp_xml x Where dbo.A_UserControlLines.UserA = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControlLines.UserA = x.a) and (A_UserControlLines.IdLine = x.b) )
			
		Insert into dbo.A_UserControlLines (UserA, IdLine)
			Select Distinct a, b
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControlLines u Where (u.UserA = x.a) and (u.IdLine = x.b) )
			
	end else begin
	
		Delete
		From dbo.A_UserControlLines
		Where exists ( Select 1 From #tmp_xml x Where dbo.A_UserControlLines.IdLine = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControlLines.IdLine = x.a) and (A_UserControlLines.UserA = x.b) )
			
		Insert into dbo.A_UserControlLines (UserA, IdLine)
			Select Distinct b, a
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControlLines u Where (u.UserA = x.b) and (u.IdLine = x.a) )
	
	end
	
end
GO

-----------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_UserOutcomingLines_Save' ))
	Drop procedure dbo.A_UserOutcomingLines_Save
GO

Create procedure dbo.A_UserOutcomingLines_Save
	@xml ntext, -- <items><a ida=""><b idb=""/><b idb=""/></a><a ida....>
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
	
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml ( idx int identity (1,1), a uniqueidentifier, b uniqueidentifier )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select ida, idb		
		From OpenXml (@idoc, '/items/a/b', 2)	
		With ( 
			ida varchar(50) '../@ida',
			idb varchar(50) '@idb'
		)	
		

	Delete
	From dbo.A_UserOutcomingLines		
	Where exists ( Select 1 From #tmp_xml x Where dbo.A_UserOutcomingLines.IdUser = x.a )			
		
	Insert into dbo.A_UserOutcomingLines (IdUser, IdLine, LineIndex)
		Select Distinct a, b, idx
		From #tmp_xml x			
	
end
GO

