﻿use oktell
GO

if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Settings' ))
	Drop view dbo.A_Settings
GO

Create view dbo.A_Settings
as
	Select * 
	From oktell_settings.dbo.A_Settings
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Settings_SaveMulti' ))
	Drop procedure dbo.A_Settings_SaveMulti
GO

Create Procedure [dbo].[A_Settings_SaveMulti]
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
	
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		name nvarchar(255) collate database_default, 
		i int,
		s nvarchar(4000) collate database_default,
		d nvarchar(100) collate database_default,
		dbl float,
		dt datetime
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xml
	
	Insert into #tmp_xml (name, i, s, d, dt)
		Select name, i, s, d, dt
		From OpenXml (@idoc1, '/items/s', 2)	
		With ( 
			name varchar(50) '@key',
			i int '@int',
			s nvarchar(4000) '@str',
			d nvarchar(100) '@dbl',
			dt datetime '@dt'			
		)	
		
	Update #tmp_xml
	Set dbl = cast(Replace(d, ',', '.') as float)
	Where not d is NULL
	
		
	-- Удалять из исходной при отсутствии в параметре не надо, т.к. частичное сохранение
		
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.valueInt = x.i,
		w.valueStr = x.s,
		w.valueDouble = x.dbl,
		w.valueDateTime = x.dt
	From dbo.A_Settings w, #tmp_xml x
	Where (w.Name = x.name)
		and ((w.valueInt != x.i)
			or (w.valueStr != x.s)
			or (w.valueDouble != x.dbl)
			or (w.valueDateTime != x.dt))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_Settings (Name, valueInt, valueStr, valueDouble, valueDateTime)
		Select Distinct Name, i, s, dbl, dt
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_Settings w 
							Where w.Name = x.name )							
end
GO

-------------------------------------------------------
--if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_AddRight' ))
--	Drop procedure dbo.[A_UserRoles_AddRight]
--GO

--if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_RemoveRight' ))
--	Drop procedure dbo.[A_UserRoles_RemoveRight]
--GO

if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_Rights_Save' ))
	Drop procedure dbo.[A_UserRoles_Rights_Save]
GO

Create procedure [dbo].[A_UserRoles_Rights_Save]
 	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
As
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
	
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		id uniqueidentifier, 
		knot nvarchar(100) collate database_default,
		name nvarchar(200) collate database_default,
		code nvarchar(100) collate database_default,
		isbasic bit
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xml
	
	return	
	
	Insert into #tmp_xml (id, knot, name, code, isbasic)
		Select id, knot, name, code, isbasic
		From OpenXml (@idoc1, '/items/upd', 2)	
		With ( 
			id varchar(50) '@id',
			knot nvarchar(100) '@knot',
			name nvarchar(200) '@name',
			code nvarchar(100) '@code',
			isbasic bit '@b'			
		)
			
	If object_id('tempdb..#tmp_xml2') is not NULL
		Drop table #tmp_xml2
	Create table #tmp_xml2 ( id uniqueidentifier )
			
	Declare @idoc2 int
	Exec sp_xml_preparedocument @idoc2 OUTPUT, @xml
		
	Insert into #tmp_xml2 (id)
		Select id
		From OpenXml (@idoc2, '/items/rem', 2)	
		With ( id varchar(50) '@id' )
		
	-- Удаляем помеченные на удаление, остальные не трогаем
	Delete 
	From dbo.A_UserRoles_Rights
	Where exists ( Select 1	
					From #tmp_xml2 x
					Where dbo.A_UserRoles_Rights.Id = x.Id )
					
	-- Обновляем те что поменялись
	Update w
	Set	w.Knot = x.knot,
		w.Name = x.name,
		w.Code = x.code,
		w.IsBasic = x.isbasic
	From dbo.A_UserRoles_Rights w, #tmp_xml x
	Where (w.Id = x.Id)
		and ((w.Knot != x.knot)
			or (w.Name != x.name)
			or (w.Code != x.code)
			or (w.IsBasic != x.isbasic))

	-- Добавляем те, которых не было
	Insert Into dbo.A_UserRoles_Rights (Id, Knot, Name, Code, IsBasic) 
		Select id, knot, name, code, isbasic
		From #tmp_xml x
		Where not exists ( Select 1
							From dbo.A_UserRoles_Rights w
							Where w.Id = x.id )

end
GO


-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_SaveRole' ))
	Drop procedure dbo.A_UserRoles_SaveRole
GO

Create procedure [dbo].A_UserRoles_SaveRole
	@id uniqueidentifier,
	@name nvarchar(200),
	@baserole uniqueidentifier = NULL,
	@domination bit,
 	@xmlrights ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
As
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Сохраняем саму роль
	if exists ( Select 1
				From dbo.A_UserRoles_Roles
				Where Id = @id ) 
	begin
				
		Update dbo.A_UserRoles_Roles
		Set 
			Id = @id,
			Name = @name,
			BaseRole = @baserole,
			Domination = @domination
		Where (Id = @id)
			and ((Name != @name)
				or (BaseRole != @baserole)
				or (Domination != @domination))

	end else begin
	
		-- Добавление новой
		Insert into dbo.A_UserRoles_Roles ( Id, Name, BaseRole, Domination ) Values ( @id, @name, @baserole, @domination )
		
		-- Заказ на пересчет кэша политики доступа
		exec A_Policy_SubjectGroupCache_SetUpdate
		
	end
	
		
	-- Обновление привилегий у роли
	if ( @xmlrights is NULL)
		return
		
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		idright uniqueidentifier,
		value int
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmlrights
	
	Insert into #tmp_xml (idright, value)
		Select idright, value
		From OpenXml (@idoc1, '/items/a', 2)	
		With ( 
			idright varchar(50) '@id',
			value int '@val'			
		)
	
	-- Удаляем те, что отсутствуют в параметре, но есть в таблице
	Delete 
	From dbo.A_UserRoles_RightsOfRoles
	Where (IdRole = @id)
		and not exists( Select 1
						From #tmp_xml x
						Where dbo.A_UserRoles_RightsOfRoles.IdRight = x.idright )
	
	-- Обновляем, те что есть в обоих местах
	Update w	
	Set w.Value = x.value
	From dbo.A_UserRoles_RightsOfRoles w, #tmp_xml x
	Where (w.IdRole = @id)
		and (w.IdRight = x.idright)
		and (w.Value != x.Value)
		
	-- Добавляем те, что есть в параметре, но нет в таблице
	Insert Into dbo.A_UserRoles_RightsOfRoles ( IdRole, IdRight, Value )
		Select @id, x.idright, x.value
		From #tmp_xml x
		Where not exists ( Select 1
							From dbo.A_UserRoles_RightsOfRoles w
							Where (w.IdRole = @id)
								and (w.IdRight = x.idright))
		
end
GO



-------------------------------------------------------
--if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_SetRoleForUser' ))
--	Drop procedure dbo.A_UserRoles_SetRoleForUser
--GO

if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_SetupRoleUsers' ))
	Drop procedure dbo.A_UserRoles_SetupRoleUsers
GO

Create procedure [dbo].A_UserRoles_SetupRoleUsers
	@idrole uniqueidentifier,	
 	@xmlusers ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
As
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
					
				
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		iduser uniqueidentifier,
		value int
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmlusers
	
	Insert into #tmp_xml (iduser, value)
		Select iduser, value
		From OpenXml (@idoc1, '/items/u', 2)	
		With ( 
			iduser varchar(50) '@id',
			value int '@val'			
		)
	
	-- Удаляем те, что присутствуют в параметре с признаком "удалено", других не трогаем, остаются как были
	Delete 
	From dbo.A_UserRoles_RolesOfUsers
	Where (IdRole = @idrole)
		and exists( Select 1
					From #tmp_xml x
					Where (dbo.A_UserRoles_RolesOfUsers.IdUser = x.iduser)
						and (x.value = 0))
		
	-- Добавляем те, что есть в параметре с признаком "добавлено", но нет в таблице
	Insert Into dbo.A_UserRoles_RolesOfUsers ( IdUser, IdRole )
		Select x.iduser, @idrole
		From #tmp_xml x
		Where (x.value = 1)
			and not exists( Select 1
							From dbo.A_UserRoles_RolesOfUsers w
							Where (w.IdRole = @idrole)
								and (w.IdUser = x.iduser))
		
end
GO

-------------------------------------------------------
--if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_RemoveRolesOfUser' ))
--	Drop procedure dbo.A_UserRoles_RemoveRolesOfUser
--GO

if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserRoles_SetupUserRoles' ))
	Drop procedure dbo.A_UserRoles_SetupUserRoles
GO

Create procedure [dbo].A_UserRoles_SetupUserRoles
	@iduser uniqueidentifier,	
 	@xmlroles ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
As
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid	
					
				
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml ( idrole uniqueidentifier )
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmlroles
	
	Insert into #tmp_xml (idrole)
		Select idrole
		From OpenXml (@idoc1, '/items/r', 2)	
		With ( idrole varchar(50) '@id' )
	
	
	-- Удаляем те, что отсутствуют в параметре
	Delete 
	From dbo.A_UserRoles_RolesOfUsers
	Where (IdUser = @iduser)
		and not exists( Select 1
						From #tmp_xml x
						Where (dbo.A_UserRoles_RolesOfUsers.IdRole = x.idrole))
		
	-- Добавляем те, что есть в параметре, но нет в таблице
	Insert Into dbo.A_UserRoles_RolesOfUsers ( IdUser, IdRole )
		Select @iduser, x.idrole
		From #tmp_xml x
		Where not exists( Select 1
							From dbo.A_UserRoles_RolesOfUsers w
							Where (w.IdUser = @iduser)
								and (w.IdRole = x.idrole))
		
end
GO

-------------------------------------------------------
if not exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_UserCurrentStates' ))	
	Create table dbo.A_UserCurrentStates
	(
		IdUser uniqueidentifier not null primary key,
		IsRedirect bit not null,
		IsCC bit not null,
		IsLunch bit not null,
		IsCCManual bit not null
	)
GO

if not exists ( Select 1 From dbo.A_UserCurrentStates ) begin
	Declare @sql nvarchar (4000)
	Set @sql ='
	Insert into A_UserCurrentStates ( IdUser, IsRedirect, IsCC, IsLunch, IsCCManual )
		Select IdUser, IsRedirect, IsCC, IsLunch, IsCCManual
		From A_UserParams
	'
	Exec sp_executesql @sql
end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_CallCenter_State_SetUserInCC' ))
	Drop procedure dbo.A_CallCenter_State_SetUserInCC
GO

Create procedure [dbo].[A_CallCenter_State_SetUserInCC]
	@iduser uniqueidentifier,
	@incc bit,
	@inlunch bit = 0
as
begin

	Update A_UserCurrentStates
	Set IsCC = @incc,
		IsLunch = @inlunch,
		IsCCManual = 0
	Where IdUser = @iduser
	
end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_CallCenter_State_SetUserInCCManual' ))
	Drop procedure dbo.A_CallCenter_State_SetUserInCCManual
GO

Create procedure [dbo].[A_CallCenter_State_SetUserInCCManual]
	@iduser uniqueidentifier,
	@inccmanual	bit
as
begin

	Update A_UserCurrentStates
	Set IsCCManual = @inccmanual
	Where IdUser = @iduser
	
end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_CallCenter_State_SetUserInLunch' ))
	Drop procedure dbo.A_CallCenter_State_SetUserInLunch
GO

Create procedure [dbo].[A_CallCenter_State_SetUserInLunch]
	@iduser uniqueidentifier,
	@inlunch bit
as
begin

	Update A_UserCurrentStates
	Set IsLunch = @inlunch
	Where IdUser = @iduser
	
end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Redirect_SetUserState' ))
	Drop procedure dbo.A_Redirect_SetUserState
GO

Create procedure [dbo].[A_Redirect_SetUserState]
	@userid uniqueidentifier,
	@isset bit
as
begin
	
	Update A_UserCurrentStates
	Set IsRedirect = @isset
	Where IdUser = @userid
	
end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Users_LoadParams' ))
	Drop procedure dbo.A_Users_LoadParams
GO

if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Users_LoadStates' ))
	Drop procedure dbo.A_Users_LoadStates
GO

Create procedure [dbo].[A_Users_LoadStates]
as
begin

	Select IdUser, IsRedirect, IsCC, IsLunch, IsCCManual 
	From A_UserCurrentStates
	
end
GO

------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Admin_UserGroup_User_Get' ))
	Drop procedure dbo.A_Admin_UserGroup_User_Get
GO

Create Procedure [dbo].[A_Admin_UserGroup_User_Get]
	@iduser uniqueidentifier
As
Begin

	Select 	u.Id, 
		u.Name, 
		u.Info, 
		u.ParentGroupId, 
		u.Login, 
		u.Password, 
		u.Type, 
		isnull ( IsOperator, 0 ) as IsOperator,		
		isnull ( up.CallerId, '' ) as CallerId,
		isnull ( u.IsDeleted, 0 ) as IsDeleted,
		u.DeleteDT,
		g.Name as GroupName,
		g.Info as GroupInfo
		
	From A_Users u
		left join A_UserParams up on u.Id = up.IdUser
		left join A_Groups g on u.ParentGroupId = g.Id
	
	Where u.Id = @iduser
End
GO

------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Admin_UserGroup_User_GetAll' ))
	Drop procedure dbo.A_Admin_UserGroup_User_GetAll
GO

Create Procedure [dbo].[A_Admin_UserGroup_User_GetAll]
As
Begin

	Select 	u.Id, 
		u.Name, 
		Info, 
		ParentGroupId, 
		Login, 
		Password, 
		Type, 
		isnull ( IsOperator, 0 ) as IsOperator,
		isnull ( up.CallerId, '' ) as CallerId,
		isnull ( u.IsDeleted, 0 ) as IsDeleted,
		u.DeleteDT
	From A_Users u
		Left join A_UserParams up on u.Id = up.IdUser
		
End
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Admin_UserGroup_User_Save' ))
	Drop procedure dbo.A_Admin_UserGroup_User_Save
GO

Create procedure [dbo].[A_Admin_UserGroup_User_Save]
	@id uniqueidentifier,
	@name nvarchar ( 200 ),
	@info nvarchar ( 1000 ),
	@parentgroupid uniqueidentifier,
	@login nvarchar ( 50 ),
	@password nvarchar ( 50 ),	
	@type int,
	@isoperator bit,
	@callerid nvarchar ( 20 ) = '',
	@isdeleted bit = 0,
	@deletedt datetime = NULL,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid


	Declare @deletedtN DateTime
	Select @deletedtN = case when (( @isdeleted is null ) or ( @isdeleted = 0 )) then NULL else @deletedt end

	-- A_Users
	if exists ( Select 1
				From A_Users
				Where Id = @id )
	begin		
		-- Обновление существующего
		Update A_Users
		Set 	
			Name 		= @name,
			Info 		= @info,
			ParentGroupId 	= @parentgroupid,
			Login		= @login,
			Password 	= @password,
			Type		= @type,
			IsDeleted	= @isdeleted,
			DeleteDT	= @deletedtN
		Where (Id = @id)
			and ((Name != @name)
				or (Info != @info)
				or (ParentGroupId != @parentgroupid)
				or (Login != @login)
				or (Password != @password)
				or (Type != @type)
				or (IsDeleted != @isdeleted)
				or (DeleteDT != @deletedtN))
				
	end else begin		
		-- Добавление нового пользователя
		Insert Into A_Users ( Id, Name, Info, ParentGroupId, Login, Password, Type, Enabled, IsDeleted, DeleteDT )
		Values ( @id, @name, @info, @parentgroupid, @login, @password, @type, 1, @isdeleted, @deletedt )
			
		-- Заказ на пересчет кэша политики доступа
		exec A_Policy_SubjectGroupCache_SetUpdate		
	end

	-- A_UserParams
	if exists ( Select 1
				From A_UserParams
				Where IdUser = @id )
	begin	
		-- Обновление существующего
		Update A_UserParams
		Set	CallerId = @callerid,
			IsOperator = @isoperator
		Where (IdUser = @id)	
			and ((CallerId != @callerid)
				or (IsOperator != @isoperator))
				
	end else begin	
		-- Добавление нового
		Insert 
		Into A_UserParams ( IdUser, IsOperator, CallerId ) 
		Values	( @id, @isoperator, @callerid )
	end
	
	-- A_UserCurrentStates
	if not exists ( Select 1
					From A_UserCurrentStates
					Where IdUser = @id ) 
	begin
		Insert into A_UserCurrentStates ( IdUser, IsRedirect, IsCC, IsLunch, IsCCManual )
		Values ( @id, 0, 0, 0, 0 )
	end


	exec A_Admin_UserGroup_Group_SiftEmpty


	-- в базе кубов поправляем имя юзера
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	If Exists ( Select * FROM oktell_cc_temp..sysobjects where id = Object_Id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorInfo' ) )
		Update oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorInfo
		Set Name = @name
		Where (Id = @id)
			and (Name != @name)

end
GO

-------------------------------------------------------------------
use oktell_settings
GO

if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserParams_tmp' ))
	Drop table dbo.A_UserParams_tmp
GO

Create table dbo.A_UserParams_tmp
(
	IdUser uniqueidentifier not null Primary key,
	IsOperator bit,
	CallerId nvarchar (100) collate database_default,
	SkillTags nvarchar(4000) collate database_default
)
GO

Insert into dbo.A_UserParams_tmp
	Select IdUser, IsOperator, CallerId, SkillTags
	From dbo.A_UserParams
GO

if (exists ( Select 1 From A_UserParams_tmp )
	or not exists( Select 1 From A_UserParams))
begin
	Exec sp_rename A_UserParams, A_UserParams_tmp2
	Exec sp_rename A_UserParams_tmp, A_UserParams
	Drop table dbo.A_UserParams_tmp2
end
GO

use oktell_settings
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserParams2' ))
	Drop table dbo.A_UserParams2
GO

use oktell
GO

if exists ( select * from sysobjects where id = object_id ( 'dbo.A_UserParams' ))
	Drop view dbo.A_UserParams
GO

Create view dbo.A_UserParams
as
	Select * 
	From oktell_settings.dbo.A_UserParams
GO


-------------------------------------------------------
-------------------------------------------------------
-------------------------------------------------------
-------------------------------------------------------
-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_DB_RemoveTails' ))
	Drop procedure dbo.A_DB_RemoveTails
GO

Create Procedure [dbo].[A_DB_RemoveTails] 
as
begin

-- Отрубание глюков после изменений в БД. Лишние ссылки, пустые элементы и т.д.

		-----------------------------------------------------
		-- преобразования на стыке версий -------------------
		-----------------------------------------------------

		Update dbo.A_ServerExtLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerGSMLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerIntLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerSipLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerUSBLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerVirtualLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		-----------------------------------------------------
		-- общие возможные глюки ----------------------------
		-----------------------------------------------------
		Delete From A_WorkPlaces Where (type in (2,3,4,6)) 
										and (parentId != NULL) 
										and (parentId != '00000000-0000-0000-0000-000000000000') 
										and not (parentId in (Select [ID] From A_WorkPlaces))
		Delete From A_WorkPlaces where (type in (2,3,4,6)) and not (LineId in ( Select [ID] From A_View_ServerIntLinesAll ))
		Delete From A_ServerExtLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerIntLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerGSMLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerUSBLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerSipLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerUSBLines where not [id] in (Select LineID From A_WORKPLACES)
		Delete From A_ServerVirtualLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		
		Delete From A_LinesReactors where not [WPID] in ( Select [id] From A_WORKPLACES )
		Delete From A_LinesReactors where not LineID in ( Select [ID] From A_View_ServerLinesAll )
		Delete From A_ServerNumbers where not [id] in ( Select [ID] From A_WorkPlaces Where type=5 )
		Delete From A_ServerNumbers where [id] in ( Select [ID] From A_WorkPlaces where hasdatabase>0 )

		Delete From A_CallRoutes_Lines where not [idroute] in ( Select [ID] From A_CallRoutes )

		Declare @t table(id_ uniqueidentifier)
		insert into @t
			Select [id] From A_WorkPlaces Union 
			Select [id] From A_Users union
			Select [id] From A_View_ServerLinesAll			

		
		Delete From A_RuleRecords where not RuleID in (Select ID From A_Rules)

		--записи правил, где пользователь или линия несуществующие
		Delete From A_RuleRecords where (InnerAddressType in (0,1)) and ( not ReactID in (Select id_ From @t ))
		--записи правил, где поток несуществующий
		Delete From A_RuleRecords where (InnerAddressType in (3)) and ( not ReactID in (Select [Id] From A_ServerStreams ))

		Delete From A_NumberPlanAction where numtype=0 and extraid not in (Select ID From A_Rules)
		Delete From A_NumberPlan where id not in (Select numid From A_NumberPlanAction)
		Delete From A_RuleRecords where InnerAddressType=2 and not ReactID in (Select ID From A_NumberPlan)
		Delete From A_Rules	Where not id in ( Select ruleid From A_RuleRecords )		
		Delete From A_UserParams Where not IdUser in ( Select Id From A_Users )		
		Delete From A_UserCurrentStates Where not IdUser in ( Select Id From A_Users )		

		Delete From A_TaskManager_TaskExtLines where not ExtLineId in ( Select id From A_ServerExtLines )
		Delete From A_TaskManager_Operators where not OperatorId in ( Select id From A_Users u inner join A_UserParams up on u.Id = up.IdUser Where up.IsOperator = 1 )
		Delete From A_TaskManager_TaskProperties where not IdTask in ( Select id From A_TaskManager_Tasks )
		Delete From A_TaskManager_Lists Where (ProjectId = '00000000-0000-0000-0000-000000000000') and not exists ( Select Id From A_TaskManager_Tasks t where t.ListId = A_TaskManager_Lists.Id )
		Delete From A_TaskManager_Tasks Where (IdProject = '00000000-0000-0000-0000-000000000000') and not exists ( Select Id From A_TaskManager_Lists t where t.Id = A_TaskManager_Tasks.ListId )

		-----------------------------------------------------
		-- сетевые адреса рабочих мест = адресу сервера - заменяем брехней 
		-----------------------------------------------------
		Declare @svraddr nvarchar ( 100 )
		Select @svraddr = Address 
		From A_WorkPlaces
		Where HasFastEngine = 1
		Update A_WorkPlaces Set Address = 'Ошибка в адресе' 
		Where Address like @svraddr and HasFastEngine = 0

		-----------------------------------------------------
		-- Null в 0 в ID юзеров по умолчанию ----------------
		-----------------------------------------------------
		Update A_ServerIntLines
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		-----------------------------------------------------
		-- в 0 несуществующих юзеров по умолчанию -----------
		-----------------------------------------------------
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users 
									Union 
									Select '00000000-0000-0000-0000-000000000000')

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users 
									Union 
									Select '00000000-0000-0000-0000-000000000000')

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users 
									Union 
									Select '00000000-0000-0000-0000-000000000000')

		-----------------------------------------------------
		-- двоение юзеров по умолчанию в ссылках по линиям --
		-----------------------------------------------------
		Declare @useddefusers table (id uniqueidentifier)
		Declare @idtozero table (id uniqueidentifier)
		Declare @lid uniqueidentifier
		Declare @uid uniqueidentifier
		Declare @tmp bit 		

		Declare defusercur Cursor For
			Select Id, DefaultUserId 
			From A_View_ServerIntLinesDefaultUsers
			Where DefaultUserId != '00000000-0000-0000-0000-000000000000'

		Open defusercur
		Fetch Next From defusercur Into @lid, @uid
		While @@FETCH_STATUS = 0
		Begin		
			Select @tmp = 1 From @useddefusers where id = @uid
			if ( @tmp > 0 )
				Insert Into @idtozero (id) values (@lid)
			else
				Insert Into @useddefusers (id) values (@uid)
			Fetch Next From defusercur  Into @lid, @uid
		End
		Close defusercur
		Deallocate defusercur
		
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerIntLines si, @idtozero iz
		Where si.Id = iz.Id

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerSipLines si, @idtozero iz
		Where si.Id = iz.Id

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerVirtualLines si, @idtozero iz
		Where si.Id = iz.Id
		
		-----------------------------------------------------
		-- удаляем несуществующие значения ------------------
		-----------------------------------------------------
		Begin Tran UIT

		Delete From A_TaskManager_UserInTask with ( TabLock, HoldLock )
		Where not exists ( Select Id From A_Users Where A_Users.Id = UserId )

		Delete From A_TaskManager_UserInTask --with ( TabLock )
		Where not exists ( Select Id From A_TaskManager_Tasks Where A_TaskManager_Tasks.Id = TaskId )
		
		Commit tran UIT


		Delete From A_TaskManager_Operators
		Where not exists ( Select Id From A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_TaskExtLines
		Where not exists ( Select Id From A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_Operators
		Where IsGroup = 0 and not Exists ( Select * From A_Users where A_Users.Id = OperatorId )
		

		Delete 
		From A_UserControls
		Where not exists (Select id From A_Users u Where u.id = A_UserControls.UserA)
			or not exists (Select id From A_Users u Where u.id = A_UserControls.UserB)
			
		Delete 		
		From A_UserControlLines
		Where not exists (Select id From A_Users u Where u.id = A_UserControlLines.UserA)			
			or (not exists (Select id From A_Users l Where l.id = A_UserControlLines.IdLine)
				and not exists (Select id From A_ServerIntLines l Where l.id = A_UserControlLines.IdLine)
				and not exists (Select id From A_ServerSipLines l Where l.id = A_UserControlLines.IdLine)
				and not exists (Select id From A_ServerExtLines l Where l.id = A_UserControlLines.IdLine)
				and not exists (Select id From A_CallRoutes l Where l.id = A_UserControlLines.IdLine)				
				and (A_UserControlLines.IdLine != '11111111-1111-1111-1111-111111111111')
				and (A_UserControlLines.IdLine != '6F5F7372-D82E-4FB6-97CB-85487A85067C')
				and (A_UserControlLines.IdLine != '14B559C8-BDF8-4D17-9968-FEB8F9F54F47'))				
				
		Delete
		From A_UserOutcomingLines
		Where not exists (Select id From A_Users u Where u.id = A_UserOutcomingLines.IdUser)
			and not exists (Select id From A_ServerExtLines l Where l.id = A_UserOutcomingLines.IdLine)
				
		-----------------------------------------------------
		-- Роли на удаленных юзеров -------------------------
		-----------------------------------------------------
		
		Delete From A_UserRoles_Roles
		Where (Domination = 1)
			and ( not Id in (Select id From A_Users))
			
		Delete
		From A_UserRoles_RolesOfUsers
		Where not IdUser in (Select Id From A_Users)
	
		Delete
		From A_UserRoles_RolesOfUsers
		Where not IdRole in (Select Id From A_UserRoles_Roles)
		
		Delete
		From A_UserRoles_RightsOfRoles
		Where not IdRole in (Select Id From A_UserRoles_Roles)
		
		Delete
		From A_UserRoles_RightsOfRoles
		Where not IdRight in (Select Id From A_UserRoles_Rights)		

		-----------------------------------------------------
		-- Редиректы на удаленных юзеров --------------------
		-----------------------------------------------------
		Delete 
		From A_Redirect
		Where IdUser not in ( Select Id From A_Users )

		-----------------------------------------------------
		-- Пустые группы, номера, несвязанные правила -------
		-----------------------------------------------------
		exec A_Admin_UserGroup_Group_SiftEmpty		

		exec A_Admin_Number_SiftByActions
		
		exec A_Admin_Rule_SiftByNumbers
		
		-----------------------------------------------------
		-- неиспользуемые плагин-модули ---------------------
		-----------------------------------------------------
		Delete From A_PluginMenu_User
		Where not exists ( Select Id From A_Users u Where u.Id = A_PluginMenu_User.IdUser )

		-----------------------------------------------------
		-- канонические адреса ------------------------------
		-----------------------------------------------------	
		Delete from A_WorkPlace_Address
		
		-----------------------------------------------------	
		-- Очистка пустых ссылок в таблицах политики прав доступа к объектам
		-----------------------------------------------------	
		exec dbo.A_Policy_CheckFilter
end
GO


-------------------------------------------------------
if exists ( Select * from sysobjects where id=  object_id ( 'dbo.A_Admin_OfficeMap_RemoveTails' ))
	Drop procedure dbo.A_Admin_OfficeMap_RemoveTails
GO

Create Procedure [dbo].[A_Admin_OfficeMap_RemoveTails]
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid

  ----- A_LinesReactors
	Declare @t table (lineid uniqueidentifier, linewptype int, parentid uniqueidentifier)
	Insert Into @t (lineid, linewptype, parentid)
		Select wp.LineID, wp.Type, wp.ParentID 
		From A_WorkPlaces wp
			Inner join A_WorkPlaces wp2 on wp.ParentID = wp2.ID
		Where (wp.LineID != '00000000-0000-0000-0000-000000000000')
			And (wp2.Type != 0)
			
	-- Удаляем те, которых не осталось
	Delete 
	From dbo.A_LinesReactors
	Where not exists ( Select 1
						From @t t
						Where (t.lineid = dbo.A_LinesReactors.LineId))
						
	-- Изменяем
	Update w
	Set w.WPID = t.parentid,
		w.Type = t.linewptype
	From dbo.A_LinesReactors w, @t t
	Where (w.LineId = t.lineid)
		and ((w.WPID != t.parentid)
			or (w.Type != t.linewptype))

	-- Добавляем новые
	Insert Into dbo.A_LinesReactors (LineID, Type, WPID)
		Select t.lineid, t.linewptype, t.parentid
		From @t t
		Where not exists ( Select 1 
							From dbo.A_LinesReactors w
							Where (w.LineId = t.lineid))
  ----------------------

	Delete from A_WorkPlace_Address

	exec A_Admin_UserGroup_Group_SiftEmpty
		
end
GO
