﻿
---------------------------------------------------------------------------
--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_PluginMenu_SaveUserLink'))
--	Drop procedure dbo.A_PluginMenu_SaveUserLink
--GO

--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_PluginMenu_ClearUserLinks'))
--	Drop procedure dbo.A_PluginMenu_ClearUserLinks
--GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_PluginMenu_Save'))
	Drop procedure dbo.A_PluginMenu_Save
GO

Create procedure [dbo].[A_PluginMenu_Save]
	@id uniqueidentifier, 
	@orderindex int, 
	@idmodule uniqueidentifier, 
	@idform uniqueidentifier, 
	@headertext nvarchar ( 300 ),  
	@menutext nvarchar ( 300 ), 
	@visiblealldefault bit, 
	@description nvarchar(4000),
	@type int, 
	@addressstring nvarchar ( 3000 ),
	@enabled bit,
	@helplink nvarchar ( 500 ) = '',
	@xmlusers ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid


  -- Сохраняем сам пункт		
	if exists ( Select * From A_PluginMenu where Id = @id ) begin
	
		-- Достаем предыдущий индекс следования
		Declare @oldorder int
		Set @oldorder = -1
	
		Select @oldorder = OrderIndex 
		From A_PluginMenu
		Where Id = @id
		
		-- Смещаем индексы следования у прочих пунктов
		if ( @orderindex > @oldorder ) begin		
			-- Сместился вперед
			Update A_PluginMenu
			Set OrderIndex = OrderIndex - 1
			Where (Id != @id)
				and (OrderIndex > @oldorder)
				and (OrderIndex <= @orderindex)
				
		end else begin
			-- Сместился назад
			Update A_PluginMenu
			Set OrderIndex = OrderIndex + 1
			Where (Id != @id)
				and (OrderIndex >= @orderindex)
				and (OrderIndex < @oldorder)
		end

		-- Обновляем данные
		Update A_PluginMenu
		Set OrderIndex = @orderindex,
			IdModule = @idmodule,
			IdForm = @idform,
			HeaderText = @headertext,
			MenuText = @menutext,
			VisibleAllDefault = @visiblealldefault,
			Description = @description,
			Type = @type,
			AddressString = @addressstring,
			Enabled = @enabled, 
			HelpLink = @helplink
		Where (Id = @id)
			and ((IdModule = @idmodule)
				or (IdForm = @idform)
				or (HeaderText = @headertext)
				or (MenuText = @menutext)
				or (VisibleAllDefault = @visiblealldefault)
				or (Description = @description)
				or (Type = @type)
				or (AddressString = @addressstring)
				or (Enabled = @enabled)
				or (HelpLink = @helplink))
				
	end else begin
	
		-- Смещаем индексы следования
		if exists ( Select * From A_PluginMenu Where OrderIndex = @orderindex )
			Update A_PluginMenu
			Set OrderIndex = OrderIndex + 1
			Where OrderIndex >= @orderindex
	
		-- Вставляем новый пункт меню
		Insert 
		Into A_PluginMenu ( Id, OrderIndex, IdModule, IdForm, HeaderText, MenuText, VisibleAllDefault, Description, Type, AddressString, Enabled, HelpLink )
		Values ( @id, @orderindex, @idmodule, @idform, @headertext, @menutext, @visiblealldefault, @description, @type, @addressstring, @enabled, @helplink )
	
	end	
	
	
  -- Сохраняем привязку к пользователям
  
	if (@xmlusers is NULL)
		return
  
    If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		iduser uniqueidentifier primary key, 
		visible bit
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xmlusers
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/u', 2)	
		With ( 
			iduser varchar(50) '@id',
			visible bit '@vis'
		)	
		
				
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_PluginMenu_User
	Where (IdMenuLink = @id)
		and not exists( Select 1 
						From #tmp_xml x 
						Where (dbo.A_PluginMenu_User.IdMenuLink = @id)
							and (dbo.A_PluginMenu_User.IdUser = x.iduser))
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Visible = x.visible
	From dbo.A_PluginMenu_User w, #tmp_xml x
	Where (w.IdMenuLink = @id)
		and (w.IdUser = x.iduser)
		and (w.Visible != x.visible)
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_PluginMenu_User (IdMenuLink, IdUser, Visible)
		Select Distinct @id, x.iduser, x.visible
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_PluginMenu_User w 
							Where (w.IdMenuLink = @id)
								and (w.IdUser = x.iduser) )
			
end
GO

-------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_PluginMenu_Delete'))
	Drop procedure dbo.A_PluginMenu_Delete
GO

Create procedure [dbo].[A_PluginMenu_Delete]
	@id uniqueidentifier,
	@authorizeduserid uniqueidentifier
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid


	Declare @oldorder int
	set @oldorder = -1

	-- Достаем предыдущий индекс следования
	Select @oldorder = OrderIndex 
	From A_PluginMenu
	Where Id = @id

	if ( @@rowcount > 0 ) begin
		
		Delete From A_PluginMenu
		Where Id = @id

		Delete From A_PluginMenu_User
		Where IdMenuLink = @id

		-- Смещаем индексы следования
		Update A_PluginMenu
		Set OrderIndex = OrderIndex - 1
		Where OrderIndex > @oldorder
		
	end
	
end
GO

-------------------------------------------------------------
--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_UserResultsProject_Save'))
--	Drop procedure dbo.A_TaskManager_UserResultsProject_Save
--GO

--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_UserResultsProject_Clear'))
--	Drop procedure dbo.A_TaskManager_UserResultsProject_Clear
--GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_UserResultsProject_SaveMulti'))
	Drop procedure dbo.A_TaskManager_UserResultsProject_SaveMulti
GO

Create procedure [dbo].A_TaskManager_UserResultsProject_SaveMulti
	@idproject uniqueidentifier,	
	@xml ntext,
	@authorizeduserid uniqueidentifier
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid


	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id int,
		name nvarchar(200) collate database_default,
		isoutput bit,
		success bit,
		isdefault bit
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/ur', 2)	
		With ( 
			id int '@id',
			name nvarchar(200) '@nm',
			isoutput bit '@out',
			success bit '@scs',
			isdefault bit '@def'
		)	
		
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_TaskManager_CardUserResultsProject
	Where (ProjectId = @idproject)
		and not exists( Select 1 
						From #tmp_xml x 
						Where (dbo.A_TaskManager_CardUserResultsProject.ProjectId = @idproject)
							and (dbo.A_TaskManager_CardUserResultsProject.Id = x.id))
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Name = x.name,
		w.IsOutput = x.isoutput,
		w.Success = x.success,
		w.IsDefault = x.isdefault
	From dbo.A_TaskManager_CardUserResultsProject w, #tmp_xml x
	Where (w.ProjectId = @idproject)
		and (w.Id = x.id)
		and ((w.Name != x.name)
			or (w.IsOutput != x.isoutput)
			or (w.Success != x.success)
			or (w.IsDefault != x.isdefault))						
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_TaskManager_CardUserResultsProject (ProjectId, Id, Name, IsOutput, Success, IsDefault)
		Select Distinct @idproject, x.id, x.name, x.isoutput, x.success, x.isdefault
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_TaskManager_CardUserResultsProject w 
							Where (w.ProjectId = @idproject)
								and (w.Id = x.id))
									
end
GO



-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_Report_Save'))
	Drop procedure dbo.[A_CallCenter_Report_Save]
GO

Create Procedure [dbo].[A_CallCenter_Report_Save]
	@id uniqueidentifier,
	@stream image,
	@name nvarchar(2000) = NULL,
	@iduser uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser


	-- Сохраняет (создает или изменяет) отчет
	
	if ( exists ( Select [id] from dbo.A_CallCenter_Reports where [id] = @id )) begin
		
		-- Пользователь NULL - это 
		--    либо обновление отчета при старте системы из файла обновления
		--    либо старая версия к новой базе подцепилась
		--  и в том и в другом случае даем сохранить
		if ( not @iduser is NULL ) begin
				
			/* Проверка прав на изменение
			*/
			Declare @checkcount int
			
			exec A_Policy_SubjectPrivileges_GetForUserByObject
					@iduser = @iduser,
					@idobject = @id,
					@idprivilege = 'A7ECEDEA-76A3-4BBF-98DA-D05CD1B84F9D',
					@onlyreturncount = 1,
					@resultcount = @checkcount OUTPUT				
			
			if ( @checkcount = 0 ) begin
				-- Прав недостаточно
				Select 1
				return
			end
		end
		
		-- Обновление существующего отчета
		Update A_CallCenter_Reports
		Set [Name] = @name,
			[Stream] = @stream
		Where [Id] = @id
		
	end else begin
		
		-- Это новый отчет
		Insert into A_CallCenter_Reports ( [Id], [Name], [Stream] )
		Values ( @id, @name, @stream )
		
	end
	
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 103,
		@name = @name,
		@idParentProject = NULL,
		@actionUserId = @iduser
	
	
	-- Все удачно
	Select 0
	
end
GO


-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_Report_Delete'))
	Drop procedure dbo.[A_CallCenter_Report_Delete]
GO

Create Procedure [dbo].[A_CallCenter_Report_Delete]
	@id uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin
	
	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		
		
	-- Удаляет отчет
	
	if ( exists ( Select [id] from dbo.A_CallCenter_Reports where [id] = @id )) begin
	
		/* Проверка прав пользователя на удаление
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @iduser,
			@idobject = @id,
			@idprivilege = 'A7ECEDEA-76A3-4BBF-98DA-D05CD1B84F9D',
			@onlyreturncount = 1,				
			@resultcount = @checkcount OUTPUT
		
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end


		-- Из отчетов
		Delete 
		From A_CallCenter_Reports
		Where [Id] = @id
		
		-- Из таблиц политики прав доступа к объектам
		exec A_Policy_Objects_Remove @id
		
	end
	
	-- Все удачно
	Select 0
	
end
GO


-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_Indicator_Save'))
	Drop procedure dbo.[A_CallCenter_Indicator_Save]
GO

Create Procedure [dbo].[A_CallCenter_Indicator_Save]
	@id uniqueidentifier,
	@stream image,
	@name nvarchar(2000) = NULL,
	@iduser uniqueidentifier = NULL
as
begin
	
	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		
		
	-- Сохраняет (создает или изменяет) индикатор
	
	if ( exists ( Select [id] from dbo.A_CallCenter_Indicators where [id] = @id )) begin
		
		-- Пользователь NULL - это 
		--    либо обновление отчета при старте системы из файла обновления
		--    либо старая версия к новой базе подцепилась
		--  и в том и в другом случае даем сохранить
		if ( not @iduser is NULL ) begin
		
			/* Проверка прав на изменение
			*/
			Declare @checkcount int
			
			exec A_Policy_SubjectPrivileges_GetForUserByObject
				@iduser = @iduser,
				@idobject = @id,
				@idprivilege = 'A2E397DF-40DB-4C80-BB2D-8CD49A29E09D',
				@onlyreturncount = 1,
				@resultcount = @checkcount OUTPUT
		
			if ( @checkcount = 0 ) begin
				-- Прав недостаточно
				Select 1
				return
			end
		end
		
		-- Обновление существующего индикатора
		Update A_CallCenter_Indicators
		Set [Name] = @name,
			[Stream] = @stream
		Where [Id] = @id
		
	end else begin
		
		-- Это новый индикатор
		Insert into A_CallCenter_Indicators ( [Id], [Name], [Stream] )
		Values ( @id, @name, @stream )
		
	end
	
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 104,
		@name = @name,
		@idParentProject = NULL,
		@actionUserId = @iduser
	
	
	-- Все удачно
	Select 0	
	
end
GO


-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_Indicator_Delete'))
	Drop procedure dbo.[A_CallCenter_Indicator_Delete]
GO

Create Procedure [dbo].[A_CallCenter_Indicator_Delete]
	@id uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		
	
	-- Удаляет индикатор

	if ( exists ( Select [id] from dbo.A_CallCenter_Indicators where [id] = @id )) begin
	
		/* Проверка прав пользователя на удаление
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @iduser,
			@idobject = @id,
			@idprivilege = 'A2E397DF-40DB-4C80-BB2D-8CD49A29E09D',
			@onlyreturncount = 1,				
			@resultcount = @checkcount OUTPUT
		
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end


		-- Из индикаторов
		Delete 
		From A_CallCenter_Indicators
		Where [Id] = @id
		
		-- Из таблиц политики прав доступа к объектам
		exec A_Policy_Objects_Remove @id
		
	end
	
	-- Все удачно
	Select 0
	
end
GO


-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Counters_Create'))
	Drop procedure dbo.[A_Counters_Create]
GO

Create procedure [dbo].[A_Counters_Create]	
	@id uniqueidentifier,
	@name nvarchar ( 256 ),
	@description nvarchar ( 1000 ),
	@datecreation datetime,
	@useridcreation	uniqueidentifier,
	@datereset datetime,
	@useridreset uniqueidentifier,
	@resetvalue	int,
	@datelastmodification datetime,
	@globalvalue int,
	@localvalue	int,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @useridcreation
		

	Delete 
	From A_CallCenter_Counters
	Where [Id] = @id
	
	Insert 
	Into A_CallCenter_Counters( 
		[Id], 
		[Name], 
		[Description], 
		DateCreation, 
		UserIdCreation, 
		DateReset, 
		UserIdReset, 
		ResetValue, 
		DateLastModification,
		GlobalValue, 
		LocalValue )
	Values
		( @id,
		@name,
		@description,
		@datecreation,
		@useridcreation,
		@datereset,
		@useridreset,
		@resetvalue,
		@datelastmodification,
		@globalvalue,
		@localvalue )

end
GO


-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Counters_Delete'))
	Drop procedure dbo.[A_Counters_Delete]
GO

Create procedure [dbo].[A_Counters_Delete]	
	@id uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
	
	Delete
	From A_CallCenter_Counters
	Where [Id] = @id
	
end
GO


-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Counters_ModifyText'))
	Drop procedure dbo.[A_Counters_ModifyText]
GO

Create procedure [dbo].[A_Counters_ModifyText]	
	@id uniqueidentifier,
	@name nvarchar ( 256 ),
	@description nvarchar ( 1000 ),
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
	
	Update A_CallCenter_Counters
	Set	[Name] = @name,
		[Description] = @description
	Where ([Id] = @id)
		and (([Name] != @name)
			or ([Description] != @description))
	
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Counters_Reset'))
	Drop procedure dbo.[A_Counters_Reset]
GO

Create procedure [dbo].[A_Counters_Reset]	
	@id uniqueidentifier,	
	@datereset datetime,
	@useridreset uniqueidentifier,
	@resetvalue	int,
	@datelastmodification datetime,	
	@localvalue	int
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @useridreset
		
	
	Update A_CallCenter_Counters
	Set	DateReset = @datereset, 
		UserIdReset = @useridreset, 
		ResetValue = @resetvalue, 
		DateLastModification = @datelastmodification,
		LocalValue = @localvalue
	Where ([Id] = @id)
		and ((DateReset != @datereset)
			or (UserIdReset != @useridreset)
			or (ResetValue != @resetvalue)
			or (DateLastModification != @datelastmodification)
			or (LocalValue != @localvalue))

end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Counters_Increment'))
	Drop procedure dbo.A_Counters_Increment
GO

Create procedure [dbo].[A_Counters_Increment]
	@id uniqueidentifier,
	@date datetime
as
begin

	Update A_CallCenter_Counters
	Set [GlobalValue] = [GlobalValue] + 1,
		[LocalValue] = [LocalValue] + 1,
		[DateLastModification] = @date
	Where (Id = @id)
	
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_PlugIns_Delete'))
	Drop procedure dbo.[A_PlugIns_Delete]
GO

Create procedure [dbo].[A_PlugIns_Delete]
	@id uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL	
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Delete 
	From A_PlugIns
	Where Id = @id
	
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_PlugIns_Save'))
	Drop procedure dbo.[A_PlugIns_Save]
GO

Create procedure [dbo].[A_PlugIns_Save]
	@id uniqueidentifier,
	@type int,
	@objectname nvarchar ( 400 ) = null,
	@name nvarchar ( 400 ),
	@description nvarchar ( 4000 ) = null,
	@path nvarchar ( 1000 ) = null,
	@loadtype int = 1,
	@loadmoment int = 0,
	@destination int = 1,
	@logmode int = 0,
	@authorizeduserid uniqueidentifier = NULL	
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Update A_PlugIns
	Set 	Type = @type,
		ObjectName = @objectname,
		Name = @name,
		Description = @description,
		Path = @path,
		LoadType = @loadtype,
		LoadMoment = @loadmoment,
		Destination = @destination,
		LogMode = @logmode
	Where Id = @id

	if ( @@rowcount = 0 )
		Insert 
		Into A_PlugIns ( Id, Type, ObjectName, Name, Description, Path, LoadType, LoadMoment, Destination, LogMode )
		Values ( @id, @type, @objectname, @name, @description, @path, @loadtype, @loadmoment, @destination, @logmode )
end
GO


---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_Save'))
	Drop procedure dbo.[A_TaskManager_Project_Save]
GO

Create procedure [dbo].[A_TaskManager_Project_Save]
	@id uniqueidentifier,
	@name nvarchar ( 100 ),
	@description nvarchar ( 255 ) = '',
	@datecreation DateTime,
	@idcreator uniqueidentifier,
	@mainscript nvarchar ( 500 ) = null,
	@shared bit = 1,
	@urestype int = 0	
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @idcreator
		

	if ( exists ( Select [id] from dbo.A_TaskManager_Projects where [id] = @id )) begin
		
		/* Проверка прав на изменение
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @idcreator,
			@idobject = @id,
			@idprivilege = 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3',
			@onlyreturncount = 1,
			@resultcount = @checkcount OUTPUT
	
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end
		
		-- Обновляем существующий проект
		Update A_TaskManager_Projects
		Set	[Name] = @name,
			[Description] = @description,
			[UserResType] = @urestype
		Where (Id = @id)
			and (([Name] != @name)
				or ([Description] != @description)
				or ([UserResType] != @urestype))
	
	end else begin
	
		-- Новый проект
		Insert into A_TaskManager_Projects ( [Id], [Name], [Description], [DateCreation], [IdCreator], [MainScript], [UserResType] )
		Values ( @id, @name, @description, @datecreation, @idcreator, @mainscript, @urestype )
	
	end


	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 101,
		@name = @name,
		@idParentProject = @id,
		@actionUserId = @idcreator
	
	
	-- В базе кубов поправляем имя проекта
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	If Exists ( Select * FROM oktell_cc_temp..sysobjects where id = Object_Id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_Project' ) )
		Update oktell_cc_temp.dbo.A_Cube_CC_Cat_Project
		Set Name = @name
		Where Id = @id
	
end
GO


---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_Delete'))
	Drop procedure dbo.[A_TaskManager_Project_Delete]
GO

Create  procedure [dbo].[A_TaskManager_Project_Delete]
	@id uniqueidentifier,
	@iduser uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		
		
	if ( exists ( Select [id] from dbo.A_TaskManager_Projects where [id] = @id )) begin
	
		/* Проверка прав пользователя на удаление
		*/
		Declare @checkcount int
		
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @iduser,
			@idobject = @id,
			@idprivilege = 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3',
			@onlyreturncount = 1,
			@resultcount = @checkcount OUTPUT			
		
		if ( @checkcount = 0 ) begin
			-- Прав недостаточно
			Select 1
			return
		end


		-- Удаляем проект
		Delete from A_TaskManager_Projects
		Where Id = @id


		-- Удаляем списки
		Declare tcursor Cursor For
			Select id 
			From A_TaskManager_Lists
			Where ProjectId = @id 

		Declare @idlist uniqueidentifier

		Open tcursor
		Fetch Next From tcursor Into @idlist

		while ( @@FETCH_STATUS = 0 ) begin
			exec A_TaskManager_List_Delete @idlist

			Fetch Next From tcursor Into @idlist
		end

		Close tcursor
		Deallocate tcursor


		-- Удаляем задачи, которые к спискам не относятся
		Delete From A_TaskManager_Tasks 
		Where IdProject = @id


		-- очищаем список пользовательских состояний уникальных для проекта
		--   не надо, чтобы кубы можно было пересекать. когда нибудь потом...
		--exec A_TaskManager_UserResultsProject_Clear @id
		
		
		-- Из таблиц политики прав доступа к объектам
		exec A_Policy_Objects_Remove @id		
		-- Заказ на очистку таблиц политики доступа
		exec A_Policy_SetFilter
		
	end
	
	-- Все удачно
	Select 0

end
GO

---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_List_Save'))
	Drop procedure dbo.[A_TaskManager_List_Save]
GO

Create procedure [dbo].[A_TaskManager_List_Save]
	@id uniqueidentifier,	
	@name nvarchar ( 100 ),
	@description nvarchar ( 255 ),
	@idproject uniqueidentifier,
	@type int,
	@query nvarchar ( 1000 ),
	@joinedtable nvarchar ( 100 ),
	@getviewedfromtable bit = 1,
	@columnid nvarchar ( 100 ),
	@columnsphone nvarchar ( 1000 ),
	@columnname nvarchar ( 100 ),
	@columnoperator nvarchar ( 100 ) = '',
	@columnskilltags nvarchar ( 1000 ) = '',
	@columnsviewed nvarchar ( 1000 ),
	@columnsvieweddescription nvarchar ( 4000 ),
	@columnssqltype nvarchar ( 4000 ),
	@columnssqltypesize nvarchar ( 4000 ),
	@dbconnid int = 0,
	@dbconnstring nvarchar(1000) = '',
	@dbconnmode int = 0,
	@dbprovidername nvarchar(100) = '',
	@iduser uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		
		
	-- Сохраняет данные о таблице абонентов
	
	Declare @tp int
	Declare @jt nvarchar ( 100 )

	Select @tp = Type, @jt = JoinedTable
	From A_TaskManager_Lists 
	Where Id = @id

	declare @sql nvarchar ( 100 )
	set @sql = 'drop table ' + @jt

	-- если это локальная таблица, и изменено ее имя
	if ( @tp in ( 1, 5, 6 )) and ( @jt != @joinedtable )
		if ( exists ( Select * From sysobjects where id = object_id ( @jt ) ) )
			exec sp_executesql @sql


	if exists ( Select 1 From dbo.A_Taskmanager_Lists where Id = @id) begin
	
		Update A_Taskmanager_Lists
		Set		
			[Name]					= @name,
			[Description]			= @description,
			[ProjectId]				= @idproject,
			[Type]					= @type,
			[Query]					= @query,
			[JoinedTable]			= @joinedtable,
			[GetViewedFromTable]	= @getviewedfromtable,
			[ColumnId] 				= @columnid,
			[ColumnsPhone] 			= @columnsphone,
			[ColumnName] 			= @columnname,
			[ColumnOperator] 		= @columnoperator,
			[ColumnSkillTags] 		= @columnskilltags,
			[ColumnsViewed] 		= @columnsviewed,
			[ColumnsViewedDescription] = @columnsvieweddescription,
			[ColumnsSqlType] 		= @columnssqltype,
			[ColumnsSqlTypeSize] 	= @columnssqltypesize,
			[DbConnId]				= @dbconnid,
			[DbConnString]			= @dbconnstring,
			[DbConnMode]			= @dbconnmode,
			[DbProviderName]		= @dbprovidername
		Where (Id = @id)
			and (([Name] = @name)
				or ([Description] = @description)
				or ([ProjectId] = @idproject)
				or ([Type] = @type)
				or ([Query] = @query)
				or ([JoinedTable] = @joinedtable)
				or ([GetViewedFromTable] = @getviewedfromtable)
				or ([ColumnId] = @columnid)
				or ([ColumnsPhone] = @columnsphone)
				or ([ColumnName] = @columnname)
				or ([ColumnOperator] = @columnoperator)
				or ([ColumnSkillTags] = @columnskilltags)
				or ([ColumnsViewed] = @columnsviewed)
				or ([ColumnsViewedDescription] = @columnsvieweddescription)
				or ([ColumnsSqlType] = @columnssqltype)
				or ([ColumnsSqlTypeSize] = @columnssqltypesize)
				or ([DbConnId] = @dbconnid)
				or ([DbConnString] = @dbconnstring)
				or ([DbConnMode] = @dbconnmode)
				or ([DbProviderName] = @dbprovidername))
	
	end else begin

		-- Добавляется новый		
		Insert into A_TaskManager_Lists ( Id, Name, Description, ProjectId, Type, Query, JoinedTable, GetViewedFromTable, ColumnId, ColumnsPhone, ColumnName, ColumnOperator, ColumnSkillTags, ColumnsViewed, ColumnsViewedDescription, ColumnsSqlType, ColumnsSqlTypeSize, DbConnId, DbConnString, DbConnMode, DbProviderName )
		Values ( @id, @name, @description, @idproject, @type, @query, @joinedtable, @getviewedfromtable, @columnid, @columnsphone, @columnname, @columnoperator, @columnskilltags, @columnsviewed, @columnsvieweddescription, @columnssqltype, @columnssqltypesize, @dbconnid, @dbconnstring, @dbconnmode, @dbprovidername )
		
	end
		
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	   только если это не авто список для задач из IVR (признак - отсутствие привязки к проекту)
	*/
	if ( @idproject != '00000000-0000-0000-0000-000000000000' )
		exec A_Policy_Objects_Save
			@idObject = @id,
			@type = 107,
			@name = @name,
			@idParentProject = @idproject,
			@actionUserId = @iduser
end
GO

---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_List_Delete'))
	Drop procedure dbo.[A_TaskManager_List_Delete]
GO

Create procedure [dbo].[A_TaskManager_List_Delete]
	@id uniqueidentifier,
	@droptable bit = 1,
	@iduser uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
	

	-- Задачи обновляем
	Update A_TaskManager_Tasks
	Set ListId = '00000000-0000-0000-0000-000000000000'
	Where ListId = @id

	-- Запоминаем тип и закрепленную таблицу
	Declare @type int, @jt nvarchar ( 100 ), @idproject uniqueidentifier
	
	Select @type = t.[Type], 
		   @jt = t.[JoinedTable],
		   @idproject = t.[ProjectId]
	From A_TaskManager_Lists t
	Where t.[id] = @id
	
	-- Из таблиц
	Delete 
	From A_TaskManager_Lists
	Where Id = @id

	-- Локальную таблицу
	declare @sql nvarchar ( 100 )
	set @sql = 'drop table ' + @jt
	if ( @type in ( 1, 5, 6 ) ) and ( @droptable = 1 ) begin
		if ( exists ( Select * From sysobjects where id = object_id ( @jt ) ) )
			exec sp_executesql @sql
	end		
	
	
	-- Из таблиц политики прав доступа к объектам
	--   только если это не авто список для задач из IVR (признак - отсутствие привязки к проекту)
	if ( @idproject != '00000000-0000-0000-0000-000000000000' )
		exec A_Policy_Objects_Remove @id
	
end
GO

---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_OperatorAdd'))
	Drop procedure dbo.[A_TaskManager_Task_OperatorAdd]
GO

Create procedure [dbo].[A_TaskManager_Task_OperatorAdd]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	

	Declare @cnt int, @maxpriority int
	
	Select @cnt = Count(*)
	From A_TaskManager_Operators 
	Where (TaskId = @taskid)
		And (Operatorid = @operatorid)
			
	if ( @cnt = 0 ) begin
	
		Select @maxpriority = Max(Priority)
		From A_TaskManager_Operators 
		Where TaskId = @taskid
		Group by TaskId
	
		Insert into A_TaskManager_Operators ( TaskId, OperatorId, IsGroup, Priority )
		Values ( @taskid, @operatorid, 0, @maxpriority + 1 )
		
	end
			
end
GO

---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_OperatorRemove'))
	Drop procedure dbo.[A_TaskManager_Task_OperatorRemove]
GO

Create procedure [dbo].[A_TaskManager_Task_OperatorRemove]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid

			
	Delete
	From A_TaskManager_Operators 
	Where (TaskId = @taskid)
		And (Operatorid = @operatorid)
			
end
GO


---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ResourceSave_LinesMaxCount'))
	Drop procedure dbo.[A_TaskManager_Task_ResourceSave_LinesMaxCount]
GO

Create Procedure [dbo].[A_TaskManager_Task_ResourceSave_LinesMaxCount]
	@id uniqueidentifier,
	@linesmaxcount int,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
	
	-- Changes limit of used extlines for task (both incoming and outcoming)
	-- Value -1 sets unlimited.
	
	Update A_TaskManager_Tasks
	Set IncomingCallsMaxCount = @linesmaxcount
	Where (Id = @id)
		and (IncomingCallsMaxCount != @linesmaxcount)
	
end
GO

---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ExtlineAdd'))
	Drop procedure dbo.[A_TaskManager_Task_ExtlineAdd]
GO

Create procedure [dbo].[A_TaskManager_Task_ExtlineAdd]
	@taskid uniqueidentifier,
	@lineid uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
			
	Select TaskId, ExtLineId 
	From A_TaskManager_TaskExtLines 
	Where TaskId = @taskid
		And ExtLineId = @lineid
		
	if ( @@rowcount = 0 )
		Insert into A_TaskManager_TaskExtLines ( TaskId, ExtLineId )
		Values ( @taskid, @lineid )
			
end
GO


---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ExtlineRemove'))
	Drop procedure dbo.[A_TaskManager_Task_ExtlineRemove]
GO

Create procedure [dbo].[A_TaskManager_Task_ExtlineRemove]
	@taskid uniqueidentifier,
	@lineid uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
				
	Delete
	From A_TaskManager_TaskExtLines 
	Where TaskId = @taskid
		And ExtLineId = @lineid
			
end
GO