﻿USE [oktell];
GO

IF OBJECT_ID('dbo.H_Authorizeduserid_SPID') IS NOT NULL
    DROP TABLE dbo.H_Authorizeduserid_SPID
GO

CREATE TABLE  dbo.H_Authorizeduserid_SPID 
(
	[authorizeduser_name] nVARCHAR(500) collate database_default, 
	[getdate] DATETIME default getdate(), 
	[tr_id] varchar(255) collate database_default, 
	[sql_text] nVARCHAR(4000) collate database_default
)
CREATE INDEX  H_Authorizeduserid_SPID_tr_id_indx ON dbo.H_Authorizeduserid_SPID ([tr_id] DESC)
CREATE INDEX  H_Authorizeduserid_SPID_getdate_indx ON dbo.H_Authorizeduserid_SPID ([getdate] DESC)
GO

IF OBJECT_ID('dbo.H_HISTORY') IS NOT NULL
    DROP TABLE dbo.H_HISTORY
GO

CREATE TABLE [dbo].[H_HISTORY] (
    [id] uniqueidentifier,
    [type] varchar(10) NULL,
    [table_name] nvarchar(100) NULL,
    [authorizeduser_name] nvarchar(500) NULL,
    [columns_name] nvarchar(4000) NULL,
    [old_values] nvarchar(4000) NULL,
    [new_values] nvarchar(4000) NULL,
    [side] smallint NULL,
    [getdate] datetime NULL DEFAULT (getdate()),
    [full_query_transaction] nvarchar(4000) NULL,
    [full_query_transactionT] nvarchar(4000) NULL,
    tr_id varchar(255),
    col_id varchar(500)
)

CREATE INDEX  H_HISTORY_type_getdate_indx ON H_HISTORY ([type], [getdate] DESC)
GO 


IF OBJECT_ID('dbo.H_Authorizeduserid_SPID_Save') IS NOT NULL
    DROP PROCEDURE dbo.H_Authorizeduserid_SPID_Save
GO
   
CREATE PROCEDURE dbo.H_Authorizeduserid_SPID_Save 
    @authorizeduserid uniqueidentifier = NULL
AS
BEGIN 

    Declare @authorizeduser_name nvarchar(500), 
		@tr_id varchar(255), 
		@sql_text nvarchar(4000), 
		@sql_handle varbinary(255)
    
    Set @authorizeduser_name = Coalesce((Select [name] From A_Users Where id = @authorizeduserid), SUSER_SNAME())
    
    IF ((Select Cast(Replace(left(Cast(SERVERPROPERTY('productversion') as varchar),2),'.','') as INT)  ) > 8) BEGIN
        
        Set @tr_id = ( Select Cast(transaction_id as varchar(255)) 
						From sys.dm_tran_current_transaction )
        
        Select top 1 @sql_handle = r.sql_handle
		From  sys.dm_exec_requests r 
        Where r.transaction_id = @tr_id
                
        Select top 1 @sql_text = s_est. [text] 
        From sys.dm_exec_sql_text (@sql_handle) s_est
    
    end else begin
    
        Declare @spid int,
           @stmt_start int,
           @stmt_end int

        Set @spid = @@SPID -- Fill this in
        If @@trancount > 0
            Execute sp_getbindtoken @tr_id OUTPUT
        
        Select top 1
            @sql_handle = sql_handle,
			@stmt_start = case stmt_start when 0 then 0 else stmt_start / 2 end,
			@stmt_end = case stmt_end when -1 then -1 else stmt_end / 2 end
        From master.dbo.sysprocesses
        Where spid = @spid
        Order by ecid

        Select @sql_text = 
            Substring ( text,
            		Coalesce( NullIf(@stmt_start, 0), 1),
            		Case @stmt_end
            			when -1
            				then DataLength(text)
            			else
            				(@stmt_end - @stmt_start)
            			end
            	)
        From ::fn_get_sql(@sql_handle)
        
        Create table #temp (
			[EventType] nvarchar(30) collate database_default, 
			[Parameters] int, 
			[EventInfo] nvarchar(4000) collate database_default
		)
		
        Insert into #temp
			exec sp_executesql N'DBCC inputbuffer (@@spid) WITH NO_INFOMSGS'
			
        Set @sql_text = Coalesce( NullIf(@sql_text,'') ,(Select top 1 EventInfo From #temp))
    END
    
    Insert Into [dbo].H_Authorizeduserid_SPID ([authorizeduser_name], [getdate], [tr_id], [sql_text])
    Values (@authorizeduser_name, GetDate(), @tr_id, @sql_text)
END
GO